/*
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.bloom;
import ostore.tapestry.api.TapestryRouteMsg;
import ostore.util.ByteUtils;
import ostore.util.InputBuffer;
import ostore.util.OutputBuffer;
import ostore.util.QSException;
import ostore.util.QuickSerializable;
import ostore.util.SHA1Hash;
import ostore.util.SecureHash;
import ostore.util.TypeTable;

public class 
SimulatedReadResp extends TapestryRouteMsg implements QuickSerializable {

    public int size;
    public SecureHash guid;

    public SimulatedReadResp (SecureHash dest, SecureHash guid, int size) {
	super (dest);
	this.guid = guid;
	this.size = size;
    }

    public SimulatedReadResp (InputBuffer buffer) throws QSException {
	guid = (SecureHash) buffer.nextObject ();
	size = buffer.nextInt ();
	for (int i = 0; i < size; ++i)
	{
	  try
	  {
	    buffer.nextByte ();
	  }
	  catch(ostore.util.AssertionViolatedException ave)
	  {
	      ostore.util.Carp.error("SimulatedReadResp.<init>: size==" + size + "\n" +
			 ave);
	      throw ave;
	  }
	}
    }

    public void serialize (OutputBuffer buffer) {
	buffer.add (guid);
	buffer.add (size);
	/**
	 * CHANGE by Hakim Weatherspoon
	 * Purpose: Must add 'size' of bytes to buffer as well.
	 * Comment: Sean, you must love to get me with these
	 *           obscure cornercases (please stop doing that)!
	 */
	byte b = 0;
	for(int i = 0; i < size; i++)
	  buffer.add(b);
	/** End CHANGE
         **************/
    }

    public String toString () { 
	return "(SimulatedReadResp " + (inbound ? "from " : "to ") + peer + 
	    " for " + guid + ")"; 
    }
}

