/*
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.bloom;
import ostore.util.ByteUtils;
import ostore.util.QuickSerializable;
import ostore.util.InputBuffer;
import ostore.util.NodeId;
import ostore.util.OutputBuffer;
import ostore.util.QSException;
import ostore.util.QuickSerializable;

/**
 * Globally unique update identifier.
 *
 * @author     Sean C. Rhea
 * @version    $Id: UpdateId.java,v 1.13 2002/10/31 07:53:37 jeffpang Exp $
 */
public class UpdateId implements QuickSerializable {

    public NodeId node_id;
    public long seq;

    public UpdateId (NodeId node_id, long seq) {
        this.node_id = node_id;
        this.seq = seq;
    }

    public UpdateId (InputBuffer buffer) throws QSException {
	node_id = (NodeId) buffer.nextObject ();
	seq = buffer.nextLong ();
    }

    public void serialize (OutputBuffer buffer) {
	buffer.add (node_id);
	buffer.add (seq);
    }



    public int hashCode () {
	return node_id.hashCode () ^ ((int) seq);
    }

    public boolean equals (Object other) {
	if (! (other instanceof UpdateId)) return false;
	UpdateId rhs = (UpdateId) other;
	if (seq != rhs.seq)
	    return false;
	return node_id.equals (rhs.node_id);
    }

    public String toString () {
	return "(UpdateId " + node_id + " " + seq + ")";
    }
}

