/**
 * Test.java
 *
 * Copyright (c) 2001 Regents of the University of California.
 * All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */

package ostore.dispatch;

import ostore.util.Carp;
import ostore.util.Debug;
import ostore.util.StopWatch;
import ostore.util.NodeId;
import seda.sandStorm.api.SinkIF;
import seda.sandStorm.api.SinkException;
import seda.sandStorm.api.SinkFullException;
import seda.sandStorm.api.QueueElementIF;

/**
 * A test program for the classifier.
 * TODO: add a regression test to pond/regression
 *
 * @author	Dennis Geels
 * @version $Id: Test.java,v 1.20 2002/07/22 20:54:53 srhea Exp $
 */

public class Test implements Runnable
{
    ////////////////////       CONSTANTS	////////////////////

    private static final boolean DEBUG = true;
    
    ////////////////////    STATIC VARIABLES 	////////////////////

    private static final int warm_up = 2000;
    private static final int num = 1000000;
    private static NodeId dummy;
    static {
	try {
	    dummy = new NodeId("0.0.0.0:0");
	} catch( Exception e ) {
	}
    }
    private static StopWatch stopwatch = StopWatch.getInstance( dummy );

    ////////////////////       VARIABLES	////////////////////
    
    private int id;
    private QueueElementIF event;
    private Classifier c;
    private String tag;
    
    ////////////////////  STATIC CONSTRUCTORS	////////////////////
    ////////////////////      CONSTRUCTORS	////////////////////

    public Test( int id, QueueElementIF event, Classifier c ){
	this.id = id;
	this.event = event;
	this.tag = Integer.toString( id );
	this.c = c;
    }

    ////////////////////     STATIC METHODS 	////////////////////

    public static void main( String args[] ) throws Exception
    {
	
	Classifier c;
	if( (args.length==1) && (args[0].equals("signals")) ) {
	    Sink s = new Sink(0);
	    String tag = "test";
	    // Can't test INIT without SandStorm.
	    Signal sig = Signal.getSignal( dummy );
	    sig.register( Signal.ALRM, tag, s );

	    sig.signal( Signal.ALRM, tag );
	    sig.alarm( tag, 10 );
	    sig.alarm( tag, 100 );
	    sig.alarm( tag, 1000 );
	    Object token = sig.alarm( tag, 10000 );
	    sig.cancel( token );
	    sig.cancel( token );
	    token = sig.alarm( tag, 10000 );	    
	    sig.signal( Signal.ALRM, tag );

	    sig.signal( Signal.RBLD, tag );
	    sig.register( Signal.RBLD, tag, s );
	    sig.signal( Signal.RBLD, "dummy" );
	    sig.signal( Signal.RBLD, tag );
	    sig.register( Signal.RBLD, null, s );
	    sig.signal( Signal.RBLD, "dummy" );
	    sig.signal( Signal.RBLD, tag );
	    
	} else if( (args.length==1) && (args[0].equals("resender")) ) {

	    Debug.turnOn( "Resender" );
	    Debug.turnOn( "Signal" );
	    Sink all_good = new Sink( 0 );
	    Sink all_dead = new DeadSink( 1, 1.0 );
	    Sink half_good = new DeadSink( 2, .5 );
	    
	    c = Classifier.getClassifier( dummy );
	    Filter f = new Filter();
	    f.requireType( "ostore.dispatch.Test$NamedEvent" );
	    c.subscribe( f, all_good );
	    c.subscribe( f, all_dead );
	    c.subscribe( f, half_good );

	    f = new Filter();
	    f.requireType( "ostore.dispatch.Test$NumberedEvent" );
	    c.subscribe( f, all_good );
	    c.subscribe( f, half_good );

	    final Resender r = new Resender( dummy, "test" );
	    Signal s = Signal.getSignal( dummy );
	    Sink alarm_catcher =
		new Sink( 3 ) {
			public Object enqueue_prepare(QueueElementIF[] elements)
			{
			    Carp.out( "Sink #" + tag + " caught event " +
				      elements[0] );
			    r.process_queues();
			    return elements[0];
			}
		    };
	    s.register( Signal.ALRM, "test", alarm_catcher );

	    // May fail
	    r.dispatch( new Test.NumberedEvent( 0 ) );
	    // Will fail
	    r.dispatch( new Test.NamedEvent( "1", 1 ) );
	    // May fail - retry
	    r.dispatch( new Test.NumberedEvent( 2 ), 2, 1000 );
	    r.dispatch( new Test.NumberedEvent( 3 ), 2, 1000 );
	    r.dispatch( new Test.NumberedEvent( 4 ), 2, 1000 );
	    // Will fail - retry anyway
	    Object token = r.dispatch( new Test.NamedEvent( "5", 5 ), 10, 100 );

	    Thread.currentThread().sleep( 500 );
	    r.cancel( token );
	    Thread.currentThread().sleep( 2000 );
	    QueueElementIF waste[] = r.clear( true );
	    for( int i=0; i<waste.length; i++ ) {
		Carp.out( "Waste: " + waste[i] );
	    }
	    
	} else {
	    Thread td = new Thread( new ClockTest() );
	    td.start();
	    td.join();
	    if( td != null ) return;

	    // Old test:
	    QueueElementIF event = new Test.NumberedEvent( 0 );
	    Thread t = null;
	    c = new Classifier( dummy );
	    
	    Carp.out( "*** Normal Tests ***" );
	    t = new Thread(new Test(0, event, c));
	    t.start();
	    t.join();
	    
	    c = new Classifier( dummy );
	    Carp.out( "*** SubClass Event Tests ***" );
	    event = new Test.NamedEvent( "0", 0 );
	    t = new Thread(new Test(0, event, c));
	    t.start();
	    t.join();
	    
	    c = new Classifier( dummy );
	    Carp.out( "*** Normal Tests 2 ***" );
	    event = new Test.OtherEvent( 0 );
	    t = new Thread(new Test(0, event, c));
	    t.start();
	    t.join();
	    
	    c = new Classifier( dummy );
	    Carp.out( "*** Concurrency Tests ***" );
	    t = new Thread(new Test(0, event, c));
	    //	    c = new Classifier( dummy );
	    event = new Test.NumberedEvent( 1 );
	    Thread t2 = new Thread( new Test(1, event, c));
	    //	    c = new Classifier( dummy );
	    event = new Test.NamedEvent( "2", 2 );
	    Thread t3 = new Thread( new Test(2, event, c));
	    t.start();
	    t2.start();
	    t3.start();
	    t.join();
	    t2.join();
	    t3.join();
	}
	
	Carp.out( "Test complete" );
	return;
    }
    
    //////////////////// 	METHODS		////////////////////

    public void one_loop() throws SinkFullException
    {
	double long_time = 0;
	double[] counts = new double[4]; // 0ms, 1ms, 2ms, >2ms.
	
	long time = System.currentTimeMillis();
	long last_time = time;
	long diff;
	Carp.out( tag + " Start: " + time );
	stopwatch.start( tag );
	for( int i=0; i< num; i++ ) {
	    c.dispatch( event );
	    time = System.currentTimeMillis();
	    diff = time-last_time;
	    switch( (int)diff ) {
	    case 0:
		counts[0]++;
		break;
	    case 1: counts[1]++;
		break;
	    case 2: counts[2]++;
		break;
	    default: counts[3]++;
		long_time += diff;
		break;
	    }
	    last_time = time;
	}
	time = stopwatch.split( tag );

	Carp.out( "Done: " + time + " ms: " + ((1000*time)/num) +
		  " microseconds per call" );
	for( int i=0; i<counts.length; i++ ) {
	    Carp.out( "counts[" + i + "] = " + counts[i] );
	}
	Carp.out( "long_time: " + long_time );
	double total = counts[1] + counts[2]*2 + long_time;
	Carp.out( "total time: "  + total );
	Carp.out( "average: " + total/num );
	Carp.out( "average (<= 2ms): " + (total-long_time)/num );
    }
    
    public void run()
    {
	try {
	    // Test #1: empty dispatch
	    Carp.out( "Null Test: " + num + " dispatches to no Filter" );
	    one_loop();
	    
	    // Test #2: single requirement
	    SinkIF s = new Test.Sink( id );
	    Filter f = new Filter();
	    f.requireType( event.getClass() );
	    //	    f.requireType( "ostore.dispatch.Test$NumberedEvent" );
	    c.subscribe( f, s );

	    Carp.out( "Single Filter Test: " + num +
		      " dispatches to single Filter, type requirement" );
	    one_loop();
	    
	    // Test #3: type and field requirements
	    f.requireValue( "tag", new Integer(id) );
	    c.subscribe( f, s );

	    Carp.out( "Double Filter Test: " + num +
		      " dispatches to two Filters.  Added a field requirement.");
	    one_loop();
	    
	    // Test #4: add another Sink with same type,field requirement
	    s = new Test.Sink(100+id);
	    c.subscribe( f, s );

	    Carp.out( "Double Sink Test: " + num +
		      " dispatches to two Sinks, Three Filters total." );
	    one_loop();
	    
	    // Test #5: Added Several unused Filters
	    f = new Filter();
	    //	    f.requireType( event.getClass() );
	    f.requireType( "ostore.dispatch.Test$NumberedEvent" );
	    s = new Test.Sink(10+id);
	    c.subscribe( f, s );
	    
	    f.requireValue( "tag", new Integer(-1) );
	    s = new Test.Sink(20+id);
	    c.subscribe( f, s );
	    
	    f = new Filter();
	    f.requireType( event.getClass() );
	    //	    f.requireType( "ostore.dispatch.Test$NamedEvent" );
	    f.requireValue( "name", "foo" );
	    s = new Test.Sink(30+id);
	    c.subscribe( f, s );
	    
	    f = new Filter();
	    f.requireType( event.getClass() );
	    //	    f.requireType( "ostore.dispatch.Test$NumberedEvent" );
	    f.requireValue( "tag", new Integer(-1) );
	    s = new Test.Sink(40+id);
	    c.subscribe( f, s );
	    
	    f = new Filter();
	    f.requireType( event.getClass() );
	    //	    f.requireType( "ostore.dispatch.Test$NumberedEvent" );
	    f.requireValue( "tag", new Integer(-2) );
	    s = new Test.Sink(50+id);
	    c.subscribe( f, s );
	    
	    Carp.out( "Extra Sinks Test: " + num +
		      " dispatches to eight Sinks, same three useful Filters." );
	    one_loop();
	    
	    // Test #6: add global filter
	    f = new Filter();
	    f.requireType( "seda.sandStorm.api.QueueElementIF" );
	    s = new Test.Sink(1000+id);
	    c.subscribe( f, s );
	    
	    Carp.out( "Global Filter Test: " + num +
		      " dispatches.  Added Filter on ALL events." );
	    one_loop();

	    Carp.out( "Thread " + tag + " all tests complete" );

	} catch( Exception e ) {
	    Carp.die( e );
	}   
    }
    
    ////////////////////     INNER CLASSES	////////////////////

    public static class Sink extends seda.sandStorm.core.SimpleSink
    {
	////////////////////     VARIABLES	////////////////////
	
	protected int tag;

	////////////////////    CONSTRUCTORS	////////////////////
	
	public Sink( int tag )
	{
	    this.tag = tag;
	}

	//////////////////// 	  METHODS	////////////////////

	// SinkIF interface: we don't care about most of it.
	public Object enqueue_prepare(QueueElementIF[] elements)
	    throws SinkFullException
	{
	    if( DEBUG )
		Carp.out( "Sink #" + tag + " caught event " + elements[0] );
	    return this;
	}
	public void enqueue_commit(Object key) {}

	public void enqueue(QueueElementIF element) 
	{
	    if( DEBUG )
		Carp.out( "Sink #" + tag + " caught event " + element );
	}
	public boolean enqueue_lossy(QueueElementIF element) {
	    if( DEBUG )
		Carp.out( "Sink #" + tag + " caught lossy event " + element );
	    return true;
	}
    }
    
    public static class DeadSink extends Sink
    {
	private double fraction;	// fraction that fail
	    
	////////////////////    CONSTRUCTORS	////////////////////
	
	public DeadSink( int tag, double fraction )
	{
	    super( tag );
	    this.fraction = fraction;
	}

	//////////////////// 	  METHODS	////////////////////

	// SinkIF interface: we don't care about most of it.
	public Object enqueue_prepare(QueueElementIF[] elements)
	    throws SinkFullException
	{
	    if( DEBUG )
		Carp.out( "DeadSink #" + tag + " caught event " +
			  elements[0] );

	    if( Math.random() < fraction ) {
		if( DEBUG )
		    Carp.out( "DeadSink #" + tag + " choked" );
		throw new SinkFullException();
	    }
	    return this;
	}
    }

    public static class NumberedEvent implements QueueElementIF
    {
	////////////////////     VARIABLES	////////////////////

	public int tag;
	
	////////////////////    CONSTRUCTORS	////////////////////

	public NumberedEvent( int tag )
	{
	    this.tag = tag;
	}
	
	//////////////////// 	  METHODS	////////////////////

	public String toString()
	{
	    return ("<NumberedEvent #" + tag + '>');
	}
    }
    public static class NamedEvent extends NumberedEvent
    {
	////////////////////     VARIABLES	////////////////////

	public String name;

	////////////////////    CONSTRUCTORS	////////////////////
	
	public NamedEvent( String name, int tag )
	{
	    super( tag );
	    this.name = name;
	}

	//////////////////// 	  METHODS	////////////////////

	public String toString()
	{
	    return ("<NamedEvent " + name + " #" + tag + '>');
	}
    }
    public static class OtherEvent implements QueueElementIF
    {
	////////////////////     VARIABLES	////////////////////

	public int tag;
	
	////////////////////    CONSTRUCTORS	////////////////////

	public OtherEvent( int tag )
	{
	    this.tag = tag;
	}
	
	//////////////////// 	  METHODS	////////////////////

	public String toString()
	{
	    return ("<OtherEvent #" + tag + '>');
	}
    }
}

class ClockTest implements Runnable
{
    private static NodeId dummy;
    static {
	try {
	    dummy = new NodeId("0.0.0.0:0");
	} catch( Exception e ) {
	}
    }
    private static StopWatch stopwatch = StopWatch.getInstance( dummy );
    private static final int num = 1000000;
    public void run() {
	long time;
	long last_time;
	long diff;
	for( int j=0; j<4; j++ ) {
	    double long_time = 0;
	    double[] counts = new double[4]; // 0ms, 1ms, 2ms, >2ms.
	    double rand;
	    java.util.Random random = new java.util.Random(0);
	    String tag = this.toString();

	    time = System.currentTimeMillis();
	    last_time = time;
	    Carp.out( tag + " Start: " + time );
	    stopwatch.start( tag );
	    for( int i=0; i< num; i++ ) {
		//		time = Test.time();
		rand = random.nextDouble();
		time = System.currentTimeMillis();
		diff = time-last_time;
		switch( (int)diff ) {
		case 0:
		    counts[0]++;
		    break;
		case 1: counts[1]++;
		    break;
		case 2: counts[2]++;
		    break;
		default: counts[3]++;
		    long_time += diff;
		    break;
		}
		last_time = time;
	    }
	    time = stopwatch.split( tag );
	    time -= (stopwatch.stop( tag ) - time ); // adjust for stopwatch latency.
	    Carp.out( "Done: " + time + " ms: " + ((1000*time)/num) +
		      " microseconds per call" );
	    for( int i=0; i<counts.length; i++ ) {
		Carp.out( "counts[" + i + "] = " + counts[i] );
	    }
	    Carp.out( "long_time: " + long_time );
	    double total = counts[1] + counts[2]*2 + long_time;
	    Carp.out( "total time: "  + total );
	    Carp.out( "average: " + total/num );
	    Carp.out( "average (<= 2ms): " + (total-long_time)/num );
	}
    }
}
