/**
 * DHMessage
 *
 * @author    Gifford Cheung
 * @version     $Id: DHMessage.java, v 1.0 2001/06/14 12:01:00 gkcheung Exp
 */

package ostore.mac;

import ostore.tapestry.api.*;
import ostore.util.*;
import ostore.security.*;
import seda.sandStorm.api.*;
import javax.crypto.interfaces.*;
import ostore.util.OutputBuffer;

/**
 * Message specific to Diffie-Hellman StS key agreement <br></br>
 */
public class DHMessage 
extends TapestryRouteMsg implements QuickSerializable {
    
    public static int INITIALIZE    = 0;
    public static int RESPOND       = 1;
    public static int FINALIZE      = 2;

    public QSDHPublicKey QSkey;
    public QSPublicKey sigkey;
    public QSMACType MACType;
    QuickSerializable user_data;
    QSByteArray mac;
    public int messageType; // one of INITIALIZE, RESPOND, FINALIZE

    public DHMessage(SecureHash toWhom,
                     DHPublicKey key,
                     QSPublicKey sigkey,
                     QSMACType MACType,
		     QuickSerializable user_data,
		     QSByteArray mac,
                     int messageType ) {
        super(toWhom);
        
        if (key != null) this.QSkey = new QSDHPublicKey(key);
        this.sigkey = sigkey;
        this.MACType = MACType;
        this.user_data = user_data;
        this.mac = mac;
        this.messageType = messageType;
    }

    /** 
     *  Generate DHMessage from data
     */
    public DHMessage (InputBuffer buffer) throws QSException {
	if (buffer.nextBoolean ())
	    QSkey = (QSDHPublicKey) buffer.nextObject ();
	MACType = (QSMACType) buffer.nextObject ();
	if (buffer.nextBoolean ()) 
	    sigkey = (QSPublicKey) buffer.nextObject ();
	if (buffer.nextBoolean ())  {
	    user_data = buffer.nextObject ();
	    mac = (QSByteArray) buffer.nextObject ();
	}
	messageType = buffer.nextInt ();
    }

    public void serialize (OutputBuffer buffer) {
	buffer.add (QSkey != null);
	if (QSkey != null) 
	    buffer.add (QSkey);
	buffer.add (MACType);
	buffer.add (sigkey != null);
	if (sigkey != null)
	    buffer.add (sigkey);
	buffer.add (mac != null);
	if (mac != null) {
	    buffer.add (user_data);
	    buffer.add (mac);
	}
	buffer.add (messageType);
    }

    public String toString () {
	String result = "(DHMessage ";
        switch (messageType) {
            case 0: result += "INITIALIZE"; break;
            case 1: result += "RESPOND";    break;
            case 2: result += "FINALIZE";   break;
        }
	result += (inbound ? " from " : " to ") + peer + ")";
	return result;
    }

    public void print() {
	System.out.println (toString ());
    }

    /*
    public void print() {
        byte[] temp;
        System.out.println("Printing a DHMessage");
        if (QSkey != null) {
            temp = ((DHPublicKey)QSkey.key()).getY().toByteArray();
            System.out.println("(*)QS Public Diffie Hellman: " + 
                                ByteUtils.print_bytes(temp, 0, temp.length) );
        } else {
            System.out.println("(*)QS Public Diffie Hellman: null");
        }

        if (sigkey != null) {
            System.out.println("(*)QS signature key: " + 
                                new SHA1Hash (sigkey));
        } else {
            System.out.println("(*)QS signature key: null");
        }

        if (MACType != null) {
            System.out.println("(*)MAC Type: " + MACType.algorithm());
        } else {
            System.out.println("(*)MAC Type: null");
        }

	// just print it out
        System.out.println("(*)Verify: " + user_data + " " + mac); 
        
        switch (messageType) {
            case 0: System.out.println("(*)Message Type: INITIALIZE"); break;
            case 1: System.out.println("(*)Message Type: RESPOND"); break;
            case 2: System.out.println("(*)Message Type: FINALIZE"); break;
        }
    }
    */
}
