/**
 * CloseConnectionMsg.java
 *
 * Copyright (c) 2002 Regents of the University of California.
 * All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */

package ostore.network;

import ostore.util.InputBuffer;
import ostore.util.NodeId;
import ostore.util.OutputBuffer;
import ostore.util.QSException;

/**
 * A message sent between Network stages to request/confirm the flushing
 * and closing of a connection.
 * <p>
 * This class contains no fields beyond those defined by {@link
 * NetworkMessage};  the {@link NetworkMessage#peer} field is sufficient
 * information.  
 *
 * @author	Dennis Geels
 * @version	$Id: CloseConnectionMsg.java,v 1.2 2002/10/30 01:12:32 geels Exp $
 */

public class CloseConnectionMsg extends NetworkMessage
{
    ////////////////////       CONSTANTS	////////////////////
    ////////////////////    STATIC VARIABLES 	////////////////////
    ////////////////////       VARIABLES	////////////////////
    ////////////////////  STATIC CONSTRUCTORS	////////////////////
    ////////////////////      CONSTRUCTORS	////////////////////

    public CloseConnectionMsg( NodeId src, NodeId dest ) {
	super( dest, false /* not inbound */ );
	this.sender = src;	// usually set by Classifier.
    }

    public CloseConnectionMsg( InputBuffer buffer ) throws QSException {
	super( buffer );
    }
    
    ////////////////////     STATIC METHODS 	////////////////////
    //////////////////// 	METHODS		////////////////////

    public String toString () {
	return "(CloseConnectionMsg peer=" + peer + ", sender=" + sender +
	    " " + (inbound ? "inbound" : "outbound") + ")";
    }
    
    ////////////////////     INNER CLASSES	////////////////////
}
