/**
 * DeltaMsg.java
 *
 * @author Jeremy Stribling
 * @version    $Id: DeltaMsg.java,v 1.1 2003/01/24 21:05:19 strib Exp $
 *
 * Copyright (c) 2003 Regents of the University of California.
 * All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */

package ostore.network.patchwork;
import ostore.util.NodeId;
import seda.sandStorm.api.QueueElementIF;

/**
 * A <code>DeltaMsg</code> is sent out every time Patchwork recalculates
 * its table of statistics.  It contains a <code>Delta</code> that should be
 * applied to every stage's local <code>PatchworkTable</code>, through the
 * table's <code>add_delta</code> method. 
 *
 * @author Jeremy Stribling
 * @version    $Id: DeltaMsg.java,v 1.1 2003/01/24 21:05:19 strib Exp $
 */
public class DeltaMsg implements QueueElementIF {

    public PatchworkTable.Delta delta;

    /**
     */
    public DeltaMsg (PatchworkTable.Delta delta) {
	this.delta = delta;
    }

    public Object clone () throws CloneNotSupportedException {
	DeltaMsg newone = (DeltaMsg) super.clone ();
	newone.delta = delta;
	return newone;
    }

    public String toString () {
	return "(DeltaMsg, delta " + delta + ")";
    }
}

