/**
 * ReplicaTag.java
 *
 * Copyright (c) 2001 Regents of the University of California.
 * All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */

package ostore.replica;

import ostore.util.QSInt;
import ostore.util.QSException;
import ostore.util.InputBuffer;
import ostore.util.OutputBuffer;
import ostore.tapestry.api.TapestryTag;

/**
 * The Tapestry tag published by object replicas (second-tier and clients).
 *
 * @author	Dennis Geels
 * @version	$Id: ReplicaTag.java,v 1.16 2003/02/23 23:11:41 hweather Exp $
 */

public class ReplicaTag implements TapestryTag
{
    ////////////////////       CONSTANTS	////////////////////
    ////////////////////    STATIC VARIABLES 	////////////////////
    ////////////////////       VARIABLES	////////////////////

    /**
     * The number of hops down the dissemination tree from the primary
     * replica (inner ring).
     * <p>
     * TODO: replace with more general QoS.
     */
    public QSInt depth;
    
    ////////////////////  STATIC CONSTRUCTORS	////////////////////
    ////////////////////      CONSTRUCTORS	////////////////////

    /**
     * Creates a new <code>ReplicaTag</code>.
     *
     * @param depth	The publishing node's depth from the inner ring,
     * measured in dissemination tree links.
     */
    public ReplicaTag( QSInt depth )
    {
	this.depth = depth;
    }
    
    /**
     * Constructs a <code>ReplicaTag</code> from its
     * <code>serialize</code>d form.
     */
    public ReplicaTag( InputBuffer buffer ) throws QSException
    {
	this.depth = (QSInt)buffer.nextObject();
    }
    
    ////////////////////     STATIC METHODS 	////////////////////
    //////////////////// 	METHODS		////////////////////

    /* Specified by ostore.util.QuickSerializable */
    public void serialize( OutputBuffer buffer )
    {
	buffer.add( depth );
    }
    
    /**
     * Tests whether this tag is equivalent to another.
     * Unpublishing a tag requires matching it here.
     */
    public boolean equals( Object other )
    {
	if( (other != null) && (other instanceof ReplicaTag) ) {
	    if( ((ReplicaTag)other).depth.equals( this.depth ) ) {
		return true;
	    }
	}
	return false;
    }

    /** Specified by java.lang.Object */
    public String toString () { return "(ReplicaTag)"; }    
    ////////////////////     INNER CLASSES	////////////////////
}
