/**
 * QSRsaPrivateKey.
 *
 * @author     Sean C. Rhea
 * @version    $Id: QSRsaPrivateKey.java,v 1.12 2002/08/02 19:39:56 geels Exp $
 */

package ostore.security;
import cryptix.jce.provider.rsa.RSAPrivateCrtKeyCryptix;
import java.math.BigInteger;
import java.security.PrivateKey;
import ostore.util.ByteUtils;
import ostore.util.Carp;
import ostore.util.QSException;
import ostore.util.OutputBuffer;
import ostore.util.InputBuffer;

/**
 * A subtype of <code>QSPrivateKey</code> for RSA private keys.
 * 
 * @author   Sean C. Rhea
 * @version  $Id: QSRsaPrivateKey.java,v 1.12 2002/08/02 19:39:56 geels Exp $
 */
public class QSRsaPrivateKey implements QSPrivateKey {

    private static final String class_tag = "QSRsaPrivateKey";

    public QSRsaPrivateKey (PrivateKey key) {
	final String tag = class_tag + ".QSRsaPrivateKey";
	if (key == null ) Carp.die( tag + ": key must be non-null.");
	if ( !(key instanceof RSAPrivateCrtKeyCryptix) )  
	    Carp.die( tag + ": key must be of type RSAPrivateCrtKeyCryptix, " +
		"not of type " + key.getClass ().getName () + ".");
	_key = (RSAPrivateCrtKeyCryptix) key;
    }

    public QSRsaPrivateKey (InputBuffer buffer) {
	BigInteger n = buffer.nextBigInteger ();
	BigInteger e = buffer.nextBigInteger ();
	BigInteger d = buffer.nextBigInteger ();
	BigInteger p = buffer.nextBigInteger ();
	BigInteger q = buffer.nextBigInteger ();
	BigInteger pep = buffer.nextBigInteger ();
	BigInteger peq = buffer.nextBigInteger ();
	BigInteger cc = buffer.nextBigInteger ();
	_key = new RSAPrivateCrtKeyCryptix (n, e, d, p, q, pep, peq, cc);
    }

    public void serialize (OutputBuffer buffer) {
        buffer.add (_key.getModulus ());
        buffer.add (_key.getPublicExponent ());
	buffer.add (_key.getPrivateExponent ());
        buffer.add (_key.getPrimeP ());
        buffer.add (_key.getPrimeQ ());
	buffer.add (_key.getPrimeExponentP ());
	buffer.add (_key.getPrimeExponentQ ());
	buffer.add (_key.getCrtCoefficient ());
    }

    public PrivateKey key () {
	return _key;
    }

    protected RSAPrivateCrtKeyCryptix _key;

}

