/**
 * ThresholdPublicKey.java
 *
 * Copyright (c) 2001 Regents of the University of California.
 * All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */

package ostore.security.thresh;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.security.PublicKey;
import ostore.security.QSPublicKey;
import ostore.security.SafeBigInt;

import ostore.util.ByteArrayOutputBuffer;
import ostore.util.ByteUtils;
import ostore.util.InputBuffer;
import ostore.util.OutputBuffer;
import ostore.util.QSException;
import ostore.util.SHA1Hash;
import ostore.util.TypeTable;

/**
 * A public key class for threshold signatures.
 *
 * @author  Sean C. Rhea
 * @version $Id: ThresholdPublicKey.java,v 1.19 2002/08/02 19:53:08 geels Exp $
 */
public class ThresholdPublicKey implements PublicKey, QSPublicKey {

    public String toString () {
	return "(ThresholdPublicKey " + new SHA1Hash (this) + ")";
    }

    private final String class_tag = "ThresholdPublicKey";

    ThresholdPublicKey (int k, int l, SafeBigInt n, SafeBigInt e) {
	this.k = k;
	this.l = l;
	this.n = n;
	this.e = e;
    }

    ///////////////// QuickSerializable Interface ////////////////////

    public ThresholdPublicKey (InputBuffer buffer) throws QSException {
	k = buffer.nextInt ();
	l = buffer.nextInt ();
	n = (SafeBigInt) buffer.nextObject ();
	e = (SafeBigInt) buffer.nextObject ();
    }

    public void serialize (OutputBuffer buffer ) {
	buffer.add (k);
	buffer.add (l);
	buffer.add (n);
	buffer.add (e);
    }

    ///////////////////// QSPublicKey Interface //////////////////////

    public PublicKey key () { return this; }

    /////////////////////////// Key Interface ////////////////////////

    public String getAlgorithm () { return "Threshold"; }
    public String getFormat () { return "Default"; }

    public byte [] getEncoded () {
	byte [] result = new byte [ByteUtils.qs_length (this)];
	ByteArrayOutputBuffer outb = new ByteArrayOutputBuffer (result);
	serialize (outb);
	return result;
    }

    /////////////////////////// Private Parts ////////////////////////

    int k, l;
    SafeBigInt n, e; 
}

