/**
 * ChannelACKMsg class.
 *
 * @author     Ling Huang
 * @version    $Id: ChannelACKMsg.java,v 1.6 2002/08/18 22:39:41 srhea Exp $
 */

package ostore.tapestry.channel;
import ostore.tapestry.impl.*;
import ostore.network.NetworkMessage;
import ostore.util.AssertionViolatedException;
import ostore.util.ByteUtils;
import ostore.util.CountBuffer;
import ostore.util.QuickSerializable;
import ostore.util.InputBuffer;
import ostore.util.NodeId;
import ostore.util.OutputBuffer;
import ostore.util.QSArray;
import ostore.util.QSByteArray;
import ostore.util.QSException;
import ostore.util.QuickSerializable;
import ostore.util.SecureHash;
import ostore.util.TypeTable;

public class ChannelACKMsg extends NetworkMessage
implements QuickSerializable
{
    public SecureHash ack_id[];	//Which message this ACK is for.
				//What should be the data type fo ACK?
//  Member functions:
//  Construction functions:
    public ChannelACKMsg()
    {
	super(null, false);
	ack_id = null;
    }

    public ChannelACKMsg(NodeId peer)
    {
	super(peer, false);
	ack_id = null;
    }

    public ChannelACKMsg(NodeId peer, SecureHash ack_id[])
    {
	super(peer, false);

	int i,num;
	num = ack_id.length;
	this.ack_id = new SecureHash[num];
	for (i = 0; i < num; i++)
	    this.ack_id[i]  = ack_id[i];
    }

    public ChannelACKMsg(NodeId peer, SecureHash ack_id)
    {
	super(peer, false);

	this.ack_id = new SecureHash[1];
	this.ack_id[0]  = ack_id;
    }

//  Deserialize functions:
    public ChannelACKMsg (InputBuffer buffer) throws QSException 
    {
	super (buffer);

	int i,num;
	num = buffer.nextInt();
	this.ack_id = new SecureHash[num];
	for (i = 0; i < num; i++)
	    ack_id[i] = (SecureHash)buffer.nextObject();
    }

//  Serialize functions:
    public void serialize (OutputBuffer buffer) 
    {
	super.serialize (buffer);

	int i,num;
	num = ack_id.length;
	buffer.add (num);
	for (i = 0; i < num; i++)
	    buffer.add (ack_id[i]);
    }

//  Functions to get member variables:
    public int get_ack_num()	             { return ack_id.length; }
    public SecureHash[] get_ack_id()         { return ack_id; }

//  Functions to set member variables:
    public void set_ack_id( SecureHash ack_id[]) 
    { 
	int i,num;
	
	num = ack_id.length;
	this.ack_id = new SecureHash[num];
	for (i = 0; i < num; i++)
	    this.ack_id[i]  = ack_id[i];
    }
    
//  Clone functions:
    public Object clone () throws CloneNotSupportedException 
    {
	ChannelACKMsg result = (ChannelACKMsg) super.clone ();

	int i, num;
	num = ack_id.length;
	result.ack_id = new SecureHash[num];
	for (i = 0; i < num; i++)
	    result.ack_id[i] = ack_id[i];

	return result;
    }

    public String toString () 
    {
	return "(ChannelACKMsg " + super.toString () + " "  + "->" + peer;
    }

    /*
    public ChannelACKMsg(NodeId peer, SecureHash src, SecureHash dest,int channel_id)
    {
	super(peer,src,dest,channel_id,-1,    // No seqence number for ack msg 
	                                0,    // First fragment for this ack msg 
	                                1     // In total only 1 fragment for ack msg );
	ack_id = null;
    }

    public ChannelACKMsg(NodeId peer, SecureHash src, SecureHash dest,int channel_id,
			 int ack_id[])
    {
	super(peer,src,dest,channel_id,-1,    // No seqence number for ack msg 
	      0,    // First fragment for this ack msg 
	      1     // In total only 1 fragment for ack msg );

	int i,num;
	num = ack_id.length;
	this.ack_id = new int[num];
	for (i = 0; i < num; i++)
	    this.ack_id[i]  = ack_id[i];
    }

    public ChannelACKMsg(NodeId peer, SecureHash src, SecureHash dest,int channel_id,
			 int ack_id[],int hop, int ttl)
    {
	super(peer,src,dest,channel_id,-1,    // No seqence number for ack msg 
	      0,    // First fragment for this ack msg 
	      1,    // In total only 1 fragment for ack msg 
	      hop,ttl);

	int i,num;
	num = ack_id.length;
	this.ack_id = new int[num];
	for (i = 0; i < num; i++)
	    this.ack_id[i]  = ack_id[i];
    }

    public ChannelACKMsg(NodeId peer, SecureHash src, SecureHash dest,int channel_id,
			 int msg_id, int frg_index,
			 int frg_num, int hop, int ttl, int ack_id[])
    {
	super(peer,src,dest,channel_id,msg_id,frg_index,frg_num,hop,ttl);

	int i,num;
	num = ack_id.length;
	this.ack_id = new int[num];
	for (i = 0; i < num; i++)
	    this.ack_id[i]  = ack_id[i];
    }
*/
}













