/**
 * DynamicDMAckMsg.java
 *
 * @author Ben Y. Zhao
 * @version $Id: DynamicDMAckMsg.java,v 1.17 2003/03/05 19:41:36 strib Exp $
 */

package ostore.tapestry.impl;
import ostore.network.NetworkMessage;
import ostore.util.NodeId;
import ostore.util.SecureHash;
import ostore.util.TypeTable;
import ostore.util.ByteUtils;
import ostore.util.QSException;
import ostore.util.OutputBuffer;
import ostore.util.QuickSerializable;
import ostore.util.InputBuffer;
import ostore.util.QuickSerializable;

/**
 * Message used by a new node to do ack a directed multicast to its
 * parent.
 */
public class DynamicDMAckMsg 
extends NetworkMessage implements QuickSerializable {

  public SecureHash newGuid;  //guid of new node
  public SecureHash src;      //guid of the sender of this msg

  public DynamicDMAckMsg (NodeId peer, SecureHash newGuid, SecureHash src) {
    super (peer, false);
    this.newGuid = newGuid;
    this.src = src;
  }

  public DynamicDMAckMsg (InputBuffer buffer) throws QSException {
    super(buffer);
    newGuid = (SecureHash) buffer.nextObject();
    src = (SecureHash) buffer.nextObject();
  }

  public void serialize (OutputBuffer buffer) {
    super.serialize (buffer);
    buffer.add(newGuid);
    buffer.add(src);
  }

  public Object clone () throws CloneNotSupportedException {
    DynamicDMAckMsg result = (DynamicDMAckMsg) super.clone();
    result.newGuid = newGuid;
    result.src = src;
    return result;
  }

  public String toString() {
    return "(DynamicDMAckMsg from:"+peer+", parent="+peer+", new Guid="
      +newGuid +")";
  }

}
