/**
 * DynamicGatewayMsg.java
 *
 * Message sent by the new node to be inserted to any gateway node in order
 * to start the insertion process.
 * @author Ben Y. Zhao
 * @version $Id: DynamicGatewayMsg.java,v 1.12 2003/03/03 22:38:38 strib Exp $
 */

package ostore.tapestry.impl;
import ostore.network.NetworkMessage;
import ostore.util.NodeId;
import ostore.util.NonceMsg;
import ostore.util.SecureHash;
import ostore.util.TypeTable;
import ostore.util.ByteUtils;
import ostore.util.QSException;
import ostore.util.OutputBuffer;
import ostore.util.InputBuffer;
import ostore.util.QuickSerializable;

public class DynamicGatewayMsg 
extends NonceMsg implements QuickSerializable {

  public SecureHash guid;
  public int digits_per_guid;
  public int digit_values;
  public int bits_per_digit;
  public boolean error;
  // the nonce the surrogate message must know about
  public int surr_nonce;

  public DynamicGatewayMsg (NodeId peer, SecureHash guid) {
    super (peer);
    this.guid = guid;
    digits_per_guid = RoutingTable.DIGITS_PER_GUID;
    digit_values = RoutingTable.DIGIT_VALUES;
    bits_per_digit = RoutingTable.BITS_PER_DIGIT;
    surr_nonce = 0;
  }


  public DynamicGatewayMsg (InputBuffer buffer) throws QSException {
    super(buffer);
    guid = (SecureHash) buffer.nextObject();
    digits_per_guid = buffer.nextInt();
    digit_values = buffer.nextInt();
    bits_per_digit = buffer.nextInt();
    error = buffer.nextBoolean();
    surr_nonce = buffer.nextInt();
  }

  public void serialize (OutputBuffer buffer) {
    super.serialize (buffer);
    buffer.add(guid);
    buffer.add(digits_per_guid);
    buffer.add(digit_values);
    buffer.add(bits_per_digit);
    buffer.add(error);
    buffer.add(surr_nonce);
  }

  public Object clone () throws CloneNotSupportedException {
    DynamicGatewayMsg result = (DynamicGatewayMsg) super.clone();
    result.guid = guid;
    return result;
  }

  public String toString() {
    return "(DynamicGatewayMsg: guid="+guid+", peer="+peer+
	", inbound="+inbound+")";
  }

}
