/**
 * DynamicPingMsg
 *
 * @author     Ben Y. Zhao
 * @version    $Id: DynamicPingMsg.java,v 1.8 2002/10/07 20:32:11 ravenben Exp $
 */

package ostore.tapestry.impl;
import ostore.network.NetworkMessage;
import ostore.util.NodeId;
import ostore.util.OutputBuffer;
import ostore.util.InputBuffer;
import ostore.util.QuickSerializable;
import ostore.util.SecureHash;
import ostore.util.QSException;

public class DynamicPingMsg 
extends NetworkMessage implements QuickSerializable {

  public int type;
  public static final int GENERAL = 0;
  /* used for general pinging and network measurement
   * The hashmap used is _out_times
   */
  public static final int NEARESTNEIGHBOR = 1;
  /* used to indicate that this is part of the nearest neighbor process
   * The hashmap used is _waiting_pings
   */
  public static final int INSERTION = 2;
  /* same behavior when handling the Pong side, but the receiver
   * of the pingmsg (should be a node in the multicast tree receiving
   * the ping from the new node) knows it's also a piggyback for
   * the fact the new node now knows about the receiver of the ping,
   * so the pinged node can send the ack upwards if unlocked and 
   * its children are acked. (See MulticastState.setNotify)
   * Hashmap used is _out_times.
   */
  public SecureHash src;

  public DynamicPingMsg (NodeId peer, SecureHash src, int type) {
    super (peer, false /* outbound */);
    this.type = type;
    this.src = src;
  }
  
  public DynamicPingMsg (InputBuffer buffer) 
    throws QSException {
    super(buffer);
    type = buffer.nextInt();
    src = (SecureHash) buffer.nextObject();
  }

  public void serialize (OutputBuffer buffer) {
    super.serialize (buffer);
    buffer.add(type);
    buffer.add(src);
  }

  public Object clone () throws CloneNotSupportedException {
    DynamicPingMsg msg = (DynamicPingMsg) super.clone ();
    msg.type = type;
    msg.src = src;
    return msg;
  }
  
  public String toString () {
    return "(DynamicPingMsg, type="+type+", src="+src+", "
      + super.toString () + ")";
  }
}

