/**
 * DynamicPtrMsg.
 *
 * A class to notify other tapestry nodes that we're pointing to them for
 * some entry in our routing table.  This message is used to build up a nodes
 * set of backpointers (_revpointers).  For a node X which we point to in
 * multiple levels of our routing table, we should send off a message for
 * each level it occurs on.
 *
 * @author     Ben Y. Zhao
 * @version $Id: DynamicPtrMsg.java,v 1.5 2003/03/05 19:41:36 strib Exp $ */

package ostore.tapestry.impl;
import ostore.network.NetworkMessage;
import ostore.util.NodeId;
import ostore.util.NonceMsg;
import ostore.util.OutputBuffer;
import ostore.util.InputBuffer;
import ostore.util.QuickSerializable;
import ostore.util.QSException;
import ostore.util.SecureHash;

public class DynamicPtrMsg 
extends NonceMsg implements QuickSerializable {

    public SecureHash srcguid;
    public int level;
    public boolean remove;

    public DynamicPtrMsg (NodeId peer, SecureHash srcguid, int level,
			  boolean remove) {
      super(peer);
      this.srcguid = srcguid;
      this.level = level;
      this.remove = remove;
    }

    public DynamicPtrMsg (NodeId peer, SecureHash srcguid, int level) {
	super (peer);
	this.srcguid = srcguid;
	this.level = level;
	this.remove = false;
    }

    public DynamicPtrMsg (InputBuffer buffer) 
    throws QSException {
      super (buffer); 
      srcguid = (SecureHash) buffer.nextObject();
      level = buffer.nextInt();
      remove = buffer.nextBoolean();
    }

    public void add_to_digest (OutputBuffer buffer) {
      throw new NoSuchMethodError ();
    }
  
    public void serialize (OutputBuffer buffer) {
      super.serialize (buffer);
      buffer.add(srcguid);
      buffer.add(level);
      buffer.add(remove);
    }

    public String toString () {
	return "(DynamicPtrMsg" + (remove?" remove":" add")+ 
	    " from " + srcguid+ ", level="+level+
	    (inbound?" from":" to") + " peer=" + peer + 
	    (inbound?" inbound":" outbound") + ")";
    }
}

