/**
 * DynamicRepairRespMsg.java<BR>
 * Message sent back after a DynamicRepairReqMsg, encapsulating a list of
 * nodes to use as alternates in the requestor's routing table.
 *
 * @author Ben Y. Zhao
 * @version $Id: DynamicRepairRespMsg.java,v 1.3 2003/03/28 00:57:00 strib Exp $
 */

package ostore.tapestry.impl;
import ostore.network.NetworkMessage;
import ostore.util.NodeId;
import ostore.util.SecureHash;
import ostore.util.QSException;
import ostore.util.Carp;
import java.util.Iterator;
import java.util.ArrayList;
import ostore.util.InputBuffer;
import ostore.util.OutputBuffer;
import ostore.util.QuickSerializable;

public class DynamicRepairRespMsg
  extends NetworkMessage implements QuickSerializable {

  public SecureHash src;  //the GUID of whoever sent this message
  public int digit;       //the digit of routing table to look at
  public int value;       //for response: the value of routing table to look at
  public int level;       //the redundant level that these backpointers are for
  public ArrayList al;    //list of possible alternates

  public DynamicRepairRespMsg (NodeId peer, SecureHash src,
			       ArrayList al, int digit, 
			       int value, int level) {
    super (peer, false /*outbound*/);
    this.src = src;
    this.al = al;
    this.digit = digit;
    this.value = value;
    this.level = level;
  }

  public DynamicRepairRespMsg (InputBuffer buffer) throws QSException {
    super(buffer) ;
    src = (SecureHash) buffer.nextObject();
    SecureHash name;
    NodeId addr;
    int size = buffer.nextInt();
    al = new ArrayList(size);
    for (int i =0 ; i < size; i++) {
      name = (SecureHash) buffer.nextObject();
      addr = (NodeId) buffer.nextObject();
      al.add(new GuidAddr(name, addr));
    }
    digit = buffer.nextInt();
    value = buffer.nextInt();
    level = buffer.nextInt();
  }

  public void serialize (OutputBuffer buffer) {
    super.serialize (buffer);
    buffer.add(src);
    buffer.add(al.size());
    
    GuidAddr ga;
    for (Iterator iter = al.iterator(); iter.hasNext(); ) {
      ga = (GuidAddr) iter.next();
      buffer.add(ga.getGuid());
      buffer.add(ga.getAddr());
    }

    buffer.add(digit);
    buffer.add(value);
    buffer.add(level);
  }
  
  public Object clone () throws CloneNotSupportedException {
    DynamicRepairRespMsg result = (DynamicRepairRespMsg) super.clone();
    result.src = src;
    result.digit = digit;
    result.value = value;
    result.level = level;
    result.al.addAll(al);
    return result;
  }

  public String toString() {
    String s = "(DynamicRepairRespMsg: from src " + src + 
	" for digit "+digit+
	" at value "+value+" at level "+level+"\n[";

    for (int i=0; i<al.size(); i++) {
      s += (GuidAddr) al.get(i) +", ";
    }
    return s;
  }

}






