/**
 * DynamicSeedMapMsg.java<BR>
 * Class to encapsulate and transfer backpointers between two nodes.  This is
 * primarily used as part of the nearest neighbor algorithm in order to
 * generate candidates for the next round of neighbor selection.
 *
 * @author Ben Y. Zhao
 * @version $Id: DynamicSeedMapMsg.java,v 1.2 2003/03/05 19:41:36 strib Exp $
 */

package ostore.tapestry.impl;
import ostore.network.NetworkMessage;
import ostore.util.NodeId;
import ostore.util.NonceAckMsg;
import ostore.util.SecureHash;
import ostore.util.QSException;
import ostore.util.Carp;
import java.util.Iterator;
import java.util.ArrayList;
import ostore.util.InputBuffer;
import ostore.util.OutputBuffer;
import ostore.util.QuickSerializable;

public class DynamicSeedMapMsg
  extends NonceAckMsg implements QuickSerializable {

  public SecureHash src;  //the GUID of whoever sent this message
  public int level;       //the level that these backpointers are for
  public ArrayList revpointers; //single level (hop) of the routing table

  public DynamicSeedMapMsg (NodeId peer, SecureHash src,
			    ArrayList revpointers, int level) {
    super (peer);
    this.src = src;
    this.revpointers = revpointers;
    this.level = level;
  }


  public DynamicSeedMapMsg (InputBuffer buffer) throws QSException {
    super(buffer) ;
    src = (SecureHash) buffer.nextObject();
    SecureHash name;
    NodeId addr;
    int size = buffer.nextInt();
    revpointers = new ArrayList(size);
    //initial capacity >= 16 / 0.75
    for (int i =0 ; i < size; i++) {
      name = (SecureHash) buffer.nextObject();
      addr = (NodeId) buffer.nextObject();
      if (!name.isNull()) 
	//if null guid, means pair of guid/addr is useless
	revpointers.add(new GuidAddr(name, addr));
      else 
	revpointers.add(new GuidAddr(null, addr));
    }
    level = buffer.nextInt();
  }

  public void serialize (OutputBuffer buffer) {
    super.serialize (buffer);
    buffer.add(src);
    buffer.add(revpointers.size());
    
    SecureHash name;
    NodeId addr;
    GuidAddr ga;
    for (Iterator iter = revpointers.iterator(); iter.hasNext(); ) {
      ga = (GuidAddr) iter.next();
      name = ga.getGuid();
      if (name == null) 
	buffer.add(src.nullHash());
      else 
	buffer.add(name);
      addr = ga.getAddr();
      if (addr == null)
	Carp.die("Should never happen!!!!");
      buffer.add(addr);
    }

    buffer.add(level);
  }
  
  public Object clone () throws CloneNotSupportedException {
    DynamicSeedMapMsg result = (DynamicSeedMapMsg) super.clone();
    result.src = src;
    result.level = level;
    result.revpointers.addAll(revpointers);
    return result;
  }

  public String toString() {
    String s = "(DynamicSeedMapMsg: for level "+level+"\n[";
    for (int i=0; i<revpointers.size(); i++) {
      s += (GuidAddr) revpointers.get(i) +", ";
    }
    return s;
  }

}
