/**
 * OverlayPath.java
 * @author     Ling Huang
 */

package ostore.tapestry.impl;
import java.util.ArrayList;
import java.util.Iterator;
import ostore.util.SecureHash;
import ostore.util.NodeId;
import ostore.util.QuickSerializable;
import ostore.util.QSException;
import ostore.util.InputBuffer;
import ostore.util.OutputBuffer;

public class OverlayPath implements QuickSerializable {
    int flag;    //0: incomplete path
                 //1: complete path
    public static final int  INCOMPLETE = 0;
    public static final int  COMPLETE   = 1;

    long start;
    public SecureHash id = null;  //Just for debug
    public SecureHash src;
    public SecureHash dst;
    public ArrayList  nodeids;
    public ArrayList  guids;
    public ArrayList  rtts;
    

    public OverlayPath(SecureHash src, SecureHash dst)
    {
	this.flag = INCOMPLETE;
	this.src  = src;
	this.dst  = dst;
	this.nodeids = new ArrayList();
	this.guids   = new ArrayList();
	this.rtts    = new ArrayList();
    }

    public OverlayPath(int flag, SecureHash src, 
		       SecureHash dst, ArrayList nodeids, 
		       ArrayList guids,  ArrayList rtts)
    {
	this.flag  = flag;
	this.src   = src;
	this.dst   = dst;
	this.guids = guids;
	this.rtts  = rtts;
    }

    public OverlayPath (InputBuffer buffer) throws QSException 
    {
	flag = buffer.nextInt();
	id   = (SecureHash) buffer.nextObject();
	src  = (SecureHash) buffer.nextObject();
	dst  = (SecureHash) buffer.nextObject();

	boolean  have_nodeids = buffer.nextBoolean();
	if ( have_nodeids )
	{
	    int size = buffer.nextInt();
	    nodeids   = new ArrayList();
	    for (int i = 0 ; i<size; i++)
		nodeids.add( (NodeId) buffer.nextObject() );
	}
	boolean  have_guids = buffer.nextBoolean();
	if ( have_guids )
	{
	    int size = buffer.nextInt();
	    guids   = new ArrayList();
	    for (int i = 0 ; i<size; i++)
		guids.add( (SecureHash) buffer.nextObject() );
	}
    }

    public void serialize (OutputBuffer buffer) 
    {
	buffer.add(flag);
	buffer.add(id);
	buffer.add(src);
	buffer.add(dst);
	if (nodeids == null)
	    buffer.add(false);
	else
	{
	    buffer.add(true);
	    buffer.add(nodeids.size());
	    Iterator it_nodeids;
	    for (it_nodeids = nodeids.iterator(); it_nodeids.hasNext();) 
	    {
		buffer.add( (NodeId)it_nodeids.next() );
	    }
	}
	if (guids == null)
	    buffer.add(false);
	else
	{
	    buffer.add(true);
	    buffer.add( guids.size() );
	    Iterator it_guids;
	    for (it_guids = guids.iterator(); it_guids.hasNext();) 
	    {
		buffer.add( (SecureHash)it_guids.next() );
	    }
	}
    }

    public String toString() 
    {
	SecureHash  hop;
	NodeId      hopid;;

	int i;
	String path_str = new String();

	path_str = "The OverlayPath with id = " + 
	           id + ": src = " + src;

	Iterator it_guids, it_nodeids;
	i = 0;
	if (guids != null)
	{
	    it_nodeids = nodeids.iterator();
	    for (it_guids = guids.iterator(); it_guids.hasNext();) 
	    {
		hop = (SecureHash) it_guids.next();
		hopid= (NodeId) it_nodeids.next();

		path_str = path_str + "; hop" + i + ": nodeid = " +
		           hopid + "; guid = " + hop; 
		i++;
	    }
	}
	path_str = path_str + "; dst = " + dst;
	return path_str;
    }

} 



