/**
 * RouteObjRootMsg.
 *
 * @author     Sean C. Rhea
 * @version    $Id: RouteObjRootMsg.java,v 1.31 2003/04/25 00:44:10 strib Exp $
 */

package ostore.tapestry.impl;
import ostore.network.NetworkMessage;
import ostore.tapestry.channel.MsgQos;
import ostore.tapestry.api.TapestryQuery;
import ostore.tapestry.api.TapestryQueryState;
import ostore.util.AssertionViolatedException;
import ostore.util.ByteUtils;
import ostore.util.CountBuffer;
import ostore.util.QuickSerializable;
import ostore.util.InputBuffer;
import ostore.util.NodeId;
import ostore.util.OutputBuffer;
import ostore.util.QSException;
import ostore.util.QuickSerializable;
import ostore.util.SecureHash;
import ostore.util.SHA1Hash;
import ostore.util.TypeTable;

public class RouteObjRootMsg 
extends TapestryMessage implements QuickSerializable {

    public SecureHash src;
    public SecureHash guid;
    public TapestryQuery query;
    public TapestryQueryState query_state;
    public SecureHash id;
    public int hop;
    public NodeId requestor;
    public int TTL;

    private QuickSerializable _user_data;
    public int _user_data_length;
    public InputBuffer _user_data_buffer;
 
    /**
     * Local optimizations flags and time-to-live
     */
    public int local_opts;
    public int local_opts_ttl;

    ///This is moved to TapestryMessage by Ling Huang 07/31/02.
    //Added by Ling Huang for reliable msg delivery.
    //protected int qos;                //The QOS of this message, should it be reliable or un-.
    //protected SecureHash msg_id;      //Unique identity for a message for retransmission.

    public QuickSerializable user_data () throws QSException {
	if (_user_data == null) {
	    _user_data = _user_data_buffer.nextObject ();
	    _user_data_buffer = null; // save memory
	}
	return _user_data;
    }

    public RouteObjRootMsg (NodeId peer, SecureHash src, SecureHash guid,
	    TapestryQuery query, TapestryQueryState query_state, 
	    SecureHash id, int hop, QuickSerializable user_data,
	    NodeId requestor) {
	super (peer, false /* outbound */);
	this.src = src;
	this.guid = guid;
	this.query = query;
	this.query_state = query_state;
	this.id = id;
	this.hop = hop;
	this.requestor = requestor;
	this.TTL = -1;
	_user_data = user_data;

	this.qos = MsgQos.UNRELIABLE;
	this.msg_id = null;
	
	local_opts = 0;
	local_opts_ttl = 0;
    }

    public RouteObjRootMsg (InputBuffer buffer) throws QSException {
	super (buffer);
	src = (SecureHash) buffer.nextObject ();
	guid = (SecureHash) buffer.nextObject ();
	if (buffer.nextBoolean ())
	    query = (TapestryQuery) buffer.nextObject ();
	if (buffer.nextBoolean ())
	    query_state = (TapestryQueryState) buffer.nextObject ();
	if (buffer.nextBoolean ())
	    id = (SecureHash) buffer.nextObject ();

	local_opts = buffer.nextInt ();
	// only if there is some optimization (saves space)
	if( local_opts > 0 ) {
	    local_opts_ttl = buffer.nextInt ();
	}

	hop = buffer.nextInt ();
	requestor = (NodeId) buffer.nextObject ();
	TTL = buffer.nextInt ();
	_user_data_length = buffer.nextInt ();
	_user_data_buffer = buffer.subBuffer (_user_data_length);
	
	/*This is moved to TapestryMessage by Ling Huang 07/31/02.
	qos = buffer.nextInt ();
	if (qos == MsgQos.RELIABLE)
	    msg_id = (SecureHash) buffer.nextObject ();
	*/
    }

    public void serialize (OutputBuffer buffer) {
	super.serialize (buffer);

	buffer.add (src);
	buffer.add (guid);

	if (query == null)
	    buffer.add (false);	
	else {
	    buffer.add (true);	
	    buffer.add (query);
	}

	if (query_state == null)
	    buffer.add (false);	
	else {
	    buffer.add (true);	
	    buffer.add (query_state);
	}

	if (id == null)
	    buffer.add (false);	
	else {
	    buffer.add (true);	
	    buffer.add (id);
	}

	buffer.add (local_opts);
	// only if there is some optimization (saves space)
	if( local_opts > 0 ) {
	    buffer.add (local_opts_ttl);
	}

	buffer.add(hop);
	buffer.add (requestor);
	buffer.add(TTL);
	if (_user_data != null) {
	    CountBuffer cb = new CountBuffer ();
	    cb.add (_user_data);
	    buffer.add (cb.size ());
	    buffer.add (_user_data);
	}
	else {
	    buffer.add (_user_data_length);
	    try {
		// In order to make the add_to_digest idempotent, I have to
		// clone the _user_data_buffer before reading the data out
		// of it.  Otherwise, it will be empty the next time this
		// function is called.
		buffer.add ((InputBuffer) _user_data_buffer.clone ());
	    }
	    catch (CloneNotSupportedException e) {
		throw new AssertionViolatedException ("must be cloneable");
	    }
	}

	/*This is moved to TapestryMessage by Ling Huang 07/31/02.
	buffer.add (qos);
	if (qos == MsgQos.RELIABLE)
	    buffer.add (msg_id);
	*/
    }

    public Object clone () throws CloneNotSupportedException {
	RouteObjRootMsg result = (RouteObjRootMsg) super.clone ();
	result.src = src;
	result.guid = guid;
	result.query = query;
	result.query_state = query_state;
	result.id = id;
	result.hop = hop;
	result.requestor = requestor;
	result.TTL = TTL;
	result._user_data = _user_data;
	result._user_data_length = _user_data_length;
	if (_user_data_buffer != null)
	    result._user_data_buffer = (InputBuffer) _user_data_buffer.clone ();

	result.local_opts = local_opts;
	result.local_opts_ttl = local_opts_ttl;

	/*This is moved to TapestryMessage by Ling Huang 07/31/02.
	result.qos = qos;
	result.msg_id = msg_id;
	*/
	return result;
    }

    /*This is moved to TapestryMessage by Ling Huang 07/31/02.
    // Added by Ling Huang for msg retransmission.
    public int get_qos()                    {   return qos; }
    public boolean is_reliable()
    {
	if (qos == MsgQos.RELIABLE)
	    return true;
	return false;
    }
    public SecureHash get_msg_id()          {   return msg_id; }
    public void set_msg_id(SecureHash id)   {   msg_id = id; }
    */

    public void set_qos(int qos)
    {   
	this.qos = qos;
	//	if (qos == MsgQos.RELIABLE && msg_id == null)
	if (msg_id == null)
	    generate_msg_id();
    }

    public SecureHash generate_msg_id()
    {
	double  id;
	String  id_string;

	id = java.lang.Math.random();
	id_string = src.toString() + guid.toString() + Double.toString(id);
	msg_id = new SHA1Hash(id_string);

	return msg_id;
    }
    
    public SecureHash generate_msg_id(Object id)
    {
	String  id_string;

	id_string = src.toString() + guid.toString() + id.toString();
	msg_id = new SHA1Hash(id_string);

	return msg_id;
    }
    
    public String toString () {
	return "(RouteObjRootMsg peer=" + peer + ", id=" + 
	    id + " " + super.toString () + ")";
    }
}














