/**
 * StaticFederation.
 *
 * @author     Sean C. Rhea
 * @version    $Id: StaticFederation.java,v 1.10 2003/04/21 23:19:44 strib Exp $
 */

package ostore.tapestry.impl;
import java.util.Map;
import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
import java.util.Iterator;
import java.util.SortedMap;
import java.util.TreeMap;
import seda.sandStorm.api.ConfigDataIF;
import seda.sandStorm.api.EventHandlerException;
import seda.sandStorm.api.EventHandlerIF;
import seda.sandStorm.api.QueueElementIF;
import ostore.network.NetworkMessage;
import ostore.util.Carp;
import ostore.util.Clock;
import ostore.util.Debug;
import ostore.util.DebugFlags;
import ostore.util.InputBuffer;
import ostore.util.OutputBuffer;
import ostore.util.QSException;
import ostore.util.QuickSerializable;
import ostore.util.SecureHash;
import ostore.dispatch.Classifier;
import ostore.dispatch.Filter;
import ostore.util.NodeId;
import seda.sandStorm.api.SinkIF;
import seda.sandStorm.api.SinkException;
import seda.sandStorm.api.ProfilableIF;
import seda.sandStorm.api.NoSuchStageException;
import seda.sandStorm.internal.SinkProxy;

public class StaticFederation implements EventHandlerIF, ProfilableIF {

    private static final String class_tag = "StaticFederation";
    private static final boolean DEBUG = DebugFlags.TAPESTRY_STATIC_FEDERATION;

    private void dispatch (QueueElementIF item) {
	final String tag = class_tag + ".dispatch";
	if (DEBUG) Debug.printtagln (tag, "dispatching " +
		item.getClass () + " event: " + item + ".");
	try {
	    Classifier.getClassifier(_self_node_id).dispatch(item);
	} catch (SinkException e) {
	    BUG (class_tag + ".dispatch: could not dispatch " + item);
	}
    }

    private static final void BUG (String msg) {
	Carp.die (msg + "  This is a bug in " +
		"ostore.tapestry.impl.StaticFederation.  " +
		"Please let Sean know about it ASAP.");
    }

    public void init (ConfigDataIF config) throws Exception {
	final String tag = class_tag + ".init";

	if (DEBUG) Debug.turnOn (class_tag);

	String stageName = config.getStage().getName();
	try 
	{
	  if(config.getBoolean("Profile"))
	  {
	    config.getManager().getProfiler().add("ostore.tapestry.impl.StaticFederation", this);
	    SinkIF sink =
	      config.getManager().getStage(stageName).getSink();
	    _sink = new SinkProxy(sink,
				  config.getManager(),
				  config.getStage().getWrapper());
	  }
	  else
	    _sink =
	      config.getManager().getStage(stageName).getSink();
	} 
	catch (NoSuchStageException e) 
	{
	  Carp.die (tag + ": could not get own sink.");
	}
	_count = config.getInt ("nodes");
	_self_node_id = new NodeId (config.getString ("node_id"));

	_all_nodes_tokens = config.getInt( "all_nodes_tokens" );
	_sim_running = config.getBoolean( "simulator_running" );

	_map = new HashMap ();
	_ready_set = new HashSet ();
	
	String [] msg_types = {
	    "ostore.tapestry.impl.StaticAliveMsg",
	    "ostore.tapestry.impl.StaticReadyMsg"
	};

	for (int i = 0; i < msg_types.length; ++i) {
	    Filter filter = new Filter ();
	    if (! filter.requireType (Class.forName (msg_types [i])))
		BUG (tag + ": could not require type " + msg_types [i]);
	    if (! filter.requireValue ("inbound", new Boolean (true)))
		BUG (tag + ": could not require inbound = true for " 
			+ msg_types [i]);
	    Classifier.getClassifier(_self_node_id).subscribe (filter, _sink);
	}
    }

    public void destroy() throws Exception {
	// TODO
    }

    //  methods for ProfilableIF
    /* Specified by seda.sandStorm.api.ProfilableIF */
    public int profileSize()
    {
	return _sink.size();
    }
    //  end methods for ProfilableIF

    public void handleEvent(QueueElementIF item) 
    throws EventHandlerException {
        final String tag = class_tag + ".handleEvent";

	if (DEBUG) Debug.printtagln (tag, "got " + item.getClass () + 
		    " event: " + item + ".");

	if (item instanceof StaticAliveMsg) {
	    StaticAliveMsg msg = (StaticAliveMsg) item;
	    if (_map.containsKey (msg.peer)) {
		if (DebugFlags.LOG_BAD_NETWORK_EVENTS) Carp.out (tag + ": "
		+
			"got duplicate map entry: " + msg.peer + " -> " +
			msg.guid + ".  Dropping it.");
		return;
	    }

	    if (_map.size () == _count) {
		if (DebugFlags.LOG_BAD_NETWORK_EVENTS) {
		    Carp.out (tag + ": got extra map entry: " + msg.peer + " -> " +
			msg.guid + ".  Dropping it.");
		    Carp.out ("Current entries are as follows:");
		    for (Iterator i = _map.keySet ().iterator (); i.hasNext (); ) {
			NodeId node_id = (NodeId) i.next ();
			SecureHash guid = (SecureHash) _map.get (node_id);
			Carp.out ("    " + node_id + " -> " + guid);
		    }
		}
		return;
	    }

	    _map.put (msg.peer, msg.guid);

	    if (_map.size () == _count) {

		if( DEBUG ) {
		    _start_all_nodes_time = 
			Clock.current_date(_self_node_id).getTime();
		}

	        if( _all_nodes_tokens > 0 ) {
		  // start up the token rings (ensure some order)
		  SortedMap smap = new TreeMap();
		  smap.putAll( _map );
		  int index = 0;
		  int increment = (int) (smap.size()/_all_nodes_tokens);
		  // if there are more tokens than nodes, make sure
		  // increment is a sane number
		  if( increment < 1 ) {
		      increment = 1;
		  }
		  for (Iterator iter = smap.keySet ().iterator (); 
		       iter.hasNext ();) {
		      NodeId currNode = (NodeId) iter.next();
		      // only send the message a certain number of people on 
		      // the list, because those nodes will forward it to 
		      // the next, etc.
		      if( index % increment == 0 ) {
			  StaticAllNodesMsg outbound = 
			    new StaticAllNodesMsg (currNode, _map);
			  if( DEBUG ) {
			      Debug.printtagln( tag, "Dispatching one " + 
						" StaticAllNodes to " 
						+ outbound.peer );
			  }
			  dispatch( outbound );
		      }
		      index++;
		  }
		} else {
		  for (Iterator i = _map.keySet ().iterator (); 
		       i.hasNext ();) {
		    NodeId n = (NodeId) i.next ();
		    StaticAllNodesMsg outbound = 
		      new StaticAllNodesMsg (n, _map);
		    dispatch (outbound);
		  }
		}
	    }
	}
	else if (item instanceof StaticReadyMsg) {
	    StaticReadyMsg msg = (StaticReadyMsg) item;

	    if (_ready_set.contains (msg.peer)) {
		if (DebugFlags.LOG_BAD_NETWORK_EVENTS) Carp.out (tag + ": "
		+
			"got duplicate ready msg from " + msg.peer + 
			".  Dropping it.");
		return;
	    }

	    if (! _map.containsKey (msg.peer)) {
		if (DebugFlags.LOG_BAD_NETWORK_EVENTS) Carp.out (tag + ": "
		+
			"got unexpected ready msg without map entry from " + 
			msg.peer + ".  Dropping it.");
		return;
	    }

	    if (_ready_set.size () == _count) {
		if (DebugFlags.LOG_BAD_NETWORK_EVENTS) Carp.out (tag + ": "
		+
			"got extra ready msg from " + msg.peer + 
			".  Dropping it.");
		return;
	    }

	    _ready_set.add (msg.peer);

	    if (_ready_set.size () == _count) {
		// we're done!
		if( DEBUG ) {
		    long total_time = 
			Clock.current_date(_self_node_id).getTime() - 
			_start_all_nodes_time;
		    Debug.printtagln( tag, "AllNodes phase took " + 
				      total_time + " ms to complete" );
		}

		for (Iterator i = _map.keySet ().iterator (); i.hasNext ();) {
		    NodeId n = (NodeId) i.next ();
		    StaticBeginMsg outbound = new StaticBeginMsg (n);
		    dispatch (outbound);
		}

		// in simulation, there's no need to run anymore
		if( _sim_running ) {
		    soss.core.Main.removeNode( _self_node_id );
		}

	    }
	}
	else {
	    BUG (tag + ": caught unexpected event " + item + ".");
	}
    }

    public void handleEvents(QueueElementIF items[]) 
	throws EventHandlerException {
	    for(int i=0; i<items.length; i++)
		handleEvent(items[i]);
	}

    private SinkIF _sink;
    private Map _map;
    private Set _ready_set;
    private int _count;
    private NodeId _self_node_id;
    private int _all_nodes_tokens;

    private long _start_all_nodes_time;
    private boolean _sim_running;
}
