/**
 * TestLocateMsg.java
 *
 * @author   Ben Y. Zhao
 * @version  $Id: TestLocateMsg.java,v 1.18 2002/11/21 02:41:29 ravenben Exp $
 *
 * Copyright (c) 2001 Regents of the University of California.
 * All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 **/

package ostore.tapestry.impl;
import ostore.util.NodeId;
import ostore.util.QSException;
import ostore.util.SecureHash;
import ostore.tapestry.api.TapestryQuery;
import ostore.tapestry.api.TapestryQueryState;
import ostore.tapestry.api.TapestryLocateMsg;
import ostore.util.InputBuffer;
import ostore.util.OutputBuffer;
import ostore.util.QuickSerializable;

public class TestLocateMsg 
extends TapestryLocateMsg implements QuickSerializable {

  NodeId querysrc;

  /**
   * Construct according to the given fields. */
  public TestLocateMsg(NodeId src, SecureHash guid, TapestryQuery query, 
		       TapestryQueryState query_state)  {
    super(guid, query, query_state, null);
    querysrc = src;
    this.id = guid.nullHash();
  }

  public TestLocateMsg(NodeId src, SecureHash guid) {
    super(guid, null, null, null);
    querysrc = src;
    this.id = guid.nullHash();
  }

  public TestLocateMsg(NodeId src, SecureHash guid, SecureHash mid) {
    super(guid, null, null, null);
    querysrc = src;
    this.id = mid;
  }

  public TestLocateMsg (InputBuffer buffer) throws QSException {
    //    super(buffer);
    id = (SecureHash) buffer.nextObject();
    querysrc = (NodeId) buffer.nextObject();
  }

  public void serialize (OutputBuffer buffer) {
    //    super.serialize (buffer);
    buffer.add(id);
    buffer.add(querysrc);
  }

  /**
   * Returns a human-readable representation of this
   * <code>TestLocateMsg</code>. */
  public String toString()
  {
    String str = new String();

    str += "<TestLocateMsg from "+querysrc+" for object " + 
      guid+","+super.toString();
    str += ">";

    return str;
  }
}

