/**
 *  ChannelTest.java
 *
 *  @author     Ling Huang
 *  @version    $Id: ChannelTest.java,v 1.7 2003/02/19 10:32:31 hweather Exp $
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. either the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.tapestry.impl.test;
import java.util.*;
import java.io.*;
import java.security.*;
import ostore.util.*;
import ostore.dispatch.*;
import ostore.tapestry.*;
import ostore.tapestry.api.*;
import ostore.tapestry.impl.*;
import seda.sandStorm.api.*;
import seda.sandStorm.core.*;
import seda.sandStorm.internal.SinkProxy;
import ostore.tapestry.channel.*;

public class ChannelTest implements EventHandlerIF, ProfilableIF {

    public String class_tag = "Test";

    private void dispatch (QueueElementIF item) 
    {
	try 
	{
	    _classifier.dispatch(item);
	} 
	catch (SinkException e) 
	{
	    Carp.out ("Could not dispatch " + item);
	    e.printStackTrace(System.err);
	    System.exit (1);
	}
    }

    public void handleEvents(QueueElementIF items[]) 
    throws EventHandlerException 
    {
	for(int i=0; i<items.length; i++)
	    handleEvent(items[i]);
    }

    public void printtagln(String tag, String msg) 
    {
	System.err.print(tag+":"+msg+"\n");
    }

    public void init (ConfigDataIF config) throws Exception 
    {

	DEBUG = true;

	Types.registerAllTypes ();

	_classifier = Classifier.getClassifier(
                new NodeId (config.getString ("node_id")));
	
	//initialize the numerical arguments

	_running = 0;
	_self_node_id = new NodeId (config.getString ("node_id"));

	class_tag = class_tag + ".["+_self_node_id.port()+"]";
	if (DEBUG) ostore.util.Debug.turnOn(class_tag);

	final String tag = class_tag +".init";
	sender = config.getString("sender").equalsIgnoreCase("true");

	sender_id = new NodeId(config.getString("sender_nodeID"));
	receiver_id = new NodeId(config.getString("receiver_nodeID"));
	receiver_guid = new SHA1Hash(receiver_id);

	String stageName = config.getStage().getName();
	try 
	{
	    if(config.getBoolean("Profile")) 
	    {
		config.getManager().getProfiler().add(
				   "ostore.tapestry.impl.test.ChannelTest", this);
		SinkIF sink = config.getManager().getStage(stageName).getSink();
		this_sink = new SinkProxy(sink,config.getManager(),
				      config.getStage().getWrapper());
	    } 
	    else
		this_sink = config.getManager().getStage(stageName).getSink();
	} 
	catch (NoSuchStageException e) 
	{
	    Carp.die (tag + ": could not get own sink.");
	}

	//	TypeTable.register_type("ostore.tapestry.impl.test.TapestryRouteMsgChannel");
	Filter filter;
	String [] event_types = 
	{
	  "seda.sandStorm.api.StagesInitializedSignal",
	  "ostore.tapestry.impl.TapestryReadyMsg"
	};

	for (int i=0; i< event_types.length; i++) 
	{
	    filter = new Filter ();
	    if (! filter.requireType (Class.forName(event_types[i]))) 
	    {
		Carp.out ("Could not require type " + event_types[i]);
		System.exit (1);
	    }
	    _classifier.subscribe (filter, this_sink);
	}

	String [] msg_types = 
	{
	    "ostore.tapestry.channel.ChannelResponseMsg",
	    //	    "ostore.tapestry.channel.ChannelRequestMsg"
	    //	    "ostore.tapestry.channel.ChannelClosedMsg"
	};

	for (int i=0; i< msg_types.length; i++) 
	{
	    filter = new Filter ();
	    if (! filter.requireType (Class.forName(msg_types[i])))
		Carp.die ("Could not require type " + msg_types[i]);
	    if (! filter.requireValue("inbound", new Boolean(true)))
		Carp.die ("Could not require inbound = true for "+
			  msg_types[i]);
	    _classifier.subscribe (filter, this_sink);
	}

	System.out.println("Tapestry ChannelTest initialized!!");
    }

    public void handleEvent(QueueElementIF item) 
	   throws EventHandlerException 
    {
	DEBUG = true;
	final String tag = class_tag+".handleEvent";
	
	if (item instanceof StagesInitializedSignal) 
	{
	    // do nothing;
	    
	} 
	else if (item instanceof TapestryReadyMsg) 
	{
	    _client_guid = ((TapestryReadyMsg)item).node_guid;
	    if (sender) 
	    {
		long   delta = 1000;
		long   start;
		int    i;

		TapestryRouteMsgChannel  msg;
		msg = new TapestryRouteMsgChannel(receiver_guid, MsgQos.RELIABLE);
		for (i = 0; i < 5; i++)
		{
		    _classifier.dispatch_later(msg, delta*i*i);
		    start = System.currentTimeMillis();
		    //System.out.println("Sending out " + i + "th reliable msg at "+ 
		    //(start+delta*i*i) + " at "+ _self_node_id);
		}

		TapestryPublishMsgChannel  pub_msg;
		TapestryTagChannel tap_tag = new TapestryTagChannel();
		pub_msg = new TapestryPublishMsgChannel(receiver_guid, tap_tag, MsgQos.RELIABLE);
		for (i = 0; i < 5; i++)
		{
		    _classifier.dispatch_later(pub_msg, delta*i*i);
		    start = System.currentTimeMillis();
		    //System.out.println("Sending out " + i + "th reliable msg at "+ 
		    //(start+delta*(i*i+30)) + " at "+ _self_node_id);
		}

		TapestryLocateMsgChannel  loc_msg;
		TapestryQueryChannel query = new TapestryQueryChannel();
		TapestryQueryStateChannel query_state = new TapestryQueryStateChannel();
		loc_msg = new TapestryLocateMsgChannel(receiver_guid, query, query_state,
			       null, MsgQos.RELIABLE);
		for (i = 0; i < 5; i++)
		{
		    _classifier.dispatch_later(loc_msg, delta*i*i);
		    start = System.currentTimeMillis();
		    //System.out.println("Sending out " + i + "th reliable msg at "+ 
		    //(start+delta*(i*i+60)) + " at "+ _self_node_id);
		}
	    } 
	    else 
	    {
		System.out.println("initializing storage arrays.");
		System.out.println("storage arrays initialized.");
	    }

	} 
	else if (item instanceof TapestryRouteMsgChannel) 
	{
		System.out.println("ostore.tapestry.impl.ChannelTest: " +
				   "received a TapestryRouteMsgChannel!");
	}
	else 
	{
	    Carp.die ("ostore.tapestry.impl.Test: Received unknown "
		      + "QueueElementIF: " + item);
	}
    }

    public void destroy() throws Exception {
	// TODO
    }

    //  methods for ProfilableIF
    /* Specified by seda.sandStorm.api.ProfilableIF */
    public int profileSize() 
    {
	return this_sink.size();
    }
    //  end methods for ProfilableIF

    public static String printGuid (SecureHash guid) 
    {
	String out="";
	int [] gdigits = RoutingTable.digits(guid);
	for (int i = 0; i < gdigits.length; ++i) 
	{
	    out += Integer.toHexString (gdigits [i]);
	}   
	return out;
    }

    private int _running;
    private boolean DEBUG;
    private boolean sender;
    private NodeId sender_id;
    private NodeId receiver_id;
    private SecureHash receiver_guid;
    private NodeId _self_node_id;
    private SecureHash _client_guid;
    private Classifier _classifier;
    private SinkIF this_sink;
    
    /*
    private ssTimer sst;
    private int warmedup;
      
    private final int INITIAL = 0;
    private final int ROUTETHROUGHPUT = 1;
    private final int LOCATETHROUGHPUT = 2;
    private final int PUBLISHTHROUGHPUT = 3;
      
    private long timer;
    private int count;
    private int currSize;
    private int currIndex;
    private int total;
    private static final int runs = 7;
      
    private MicroRouteMsg [] _messages;
    private int readycount;
    private int foundcount;
    private int failedcount;
    */
}








