/**
 * LocalOptimizationTest.java
 *
 * Copyright (c) 2001 Regents of the University of California.  All
 * rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the University nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.tapestry.impl.test;
import java.util.Random;
import java.util.HashSet;
import ostore.dispatch.Classifier;
import ostore.dispatch.Filter;
import ostore.network.NetworkMessage;
import ostore.tapestry.api.TapestryLocateFailure;
import ostore.tapestry.api.TapestryLocateMsg;
import ostore.tapestry.api.TapestryPublishMsg;
import ostore.tapestry.impl.TapestryReadyMsg;
import ostore.tapestry.impl.TestLocateMsg;
import ostore.tapestry.impl.RoutingTable;
import ostore.tapestry.impl.Router;
import ostore.util.Carp;
import ostore.util.Clock;
import ostore.util.Debug;
import ostore.util.InputBuffer;
import ostore.util.NodeId;
import ostore.util.OutputBuffer;
import ostore.util.QuickSerializable;
import ostore.util.QSException;
import ostore.util.SecureHash;
import ostore.util.SHA1Hash;
import seda.sandStorm.api.ConfigDataIF;
import seda.sandStorm.api.EventHandlerException;
import seda.sandStorm.api.EventHandlerIF;
import seda.sandStorm.api.QueueElementIF;
import seda.sandStorm.api.SingleThreadedEventHandlerIF;
import seda.sandStorm.api.SinkException;
import seda.sandStorm.api.SinkFullException;
import seda.sandStorm.api.SinkIF;
import seda.sandStorm.api.StagesInitializedSignal;

/**
 * TODO
 *
 * @author     Jeremy Stribling
 * @version    $Id: LocalOptimizationTest.java,v 1.17 2003/05/22 23:28:51 strib Exp $
 */
public class LocalOptimizationTest 
implements SingleThreadedEventHandlerIF, EventHandlerIF {

    protected String TAG = "LocalOptimizationTest";
    protected static final boolean DEBUG = true;

    protected SinkIF this_sink;
    protected boolean start;
    protected long start_time;
    protected Classifier classifier;
    protected NodeId node_id;

    // config options
    private int objtotal, objstart, objeach, objfind;
    private long random_seed;
    private int optimize_on;

    private SecureHash self_guid;

    private Random rand;

    private long lastTimeSent;
    private SecureHash lastGuid;
    private int lastIndex;
    private Object lastToken;
    private boolean stanford_test;

    private HashSet published_guids;

    public static class AlarmEvent implements QueueElementIF {}

    public void init (ConfigDataIF config) throws Exception {

	// Set up some debug stuff.

	TAG = config.getStage().getName();
        if (DEBUG) Debug.turnOn (TAG);

	// Set this_sink.

	this_sink = config.getStage().getSink();
	node_id = new NodeId (config.getString ("node_id"));
	classifier = Classifier.getClassifier (node_id);

	// config opts

	objtotal = config.getInt ("objtotal");
	objstart = config.getInt ("objstart");
	objeach = config.getInt ("objeach");
	objfind = config.getInt ("objfind");
	optimize_on = config.getInt( "optimize_on" );
	stanford_test = config.getBoolean( "stanford_test" );

	published_guids = new HashSet();

	String random_str = config.getString( "random_seed" );
	if( random_str == null ) {
	    random_seed = -1;
	} else {
	    random_seed = Long.parseLong( random_str );
	}
	rand = new Random( random_seed + node_id.hashCode() );

	// Create all of our filters.

	String [] inb_msg_types = {
	    "ostore.tapestry.impl.test.LocalOptimizationTest$LocateMsg",
	    "ostore.tapestry.impl.test.LocalOptimizationTest$Response",
	    "ostore.tapestry.api.TapestryLocateFailure"
	};

	Filter filter;
	for (int i = 0; i < inb_msg_types.length; ++i) {
	    filter = new Filter ();
	    if (! filter.requireType (Class.forName (inb_msg_types [i])))
		Carp.die ("could not require type " + inb_msg_types [i]);
	    if (! filter.requireValue ("inbound", new Boolean (true)))
		Carp.die ("could not require inbound = true for " 
			  + inb_msg_types [i]);
	    classifier.subscribe (filter, this_sink);
	}

	String [] event_types = {
	    "ostore.tapestry.impl.RoutingTable$Available",
	    "ostore.tapestry.impl.test.LocalOptimizationTest$AlarmEvent",
	    "seda.sandStorm.api.StagesInitializedSignal",
	    "ostore.tapestry.impl.TapestryReadyMsg"
	};

	for (int i = 0; i < event_types.length; ++i) {
	    filter = new Filter ();
	    if (! filter.requireType (Class.forName (event_types [i])))
		Carp.die("could not require type " + event_types [i]);
	    classifier.subscribe (filter, this_sink);
	}

    }

    public void handleEvent (QueueElementIF item) 
    throws EventHandlerException {

	if (DEBUG) 
	    if (! (item instanceof RoutingTable.Available))
		Debug.printtagln (TAG, "event " + item);

	if (item instanceof StagesInitializedSignal) {
	    // do nothing

	}
	else if( item instanceof TapestryReadyMsg ) {
	    self_guid = ((TapestryReadyMsg)item).node_guid;
	    init_publish();
	    classifier.dispatch_later (new AlarmEvent (), 60000);
	}
	else if( item instanceof RoutingTable.Available ) {
	    // do nothing
	}
	else if (item instanceof LocateMsg) {
	    LocateMsg msg = (LocateMsg) item;
	    // send a response off
	    if( published_guids.contains( msg.guid ) ) {
		dispatch( new Response( msg.peer, msg.hopCount, 
					Clock.current_date(node_id).getTime(),
					msg.guid ) );
	    } else {
		dispatch( new TapestryLocateFailure( self_guid, msg.id, 
						     msg.guid, msg.peer, 
						     msg.hopCount ) );
	    }
	}
	else if (item instanceof Response) {
	    got_resp ( (Response) item );
	    // lookup once in a while
	    classifier.dispatch_later (new AlarmEvent (), 10000);
	}
	else if (item instanceof AlarmEvent) {
	    send_new_msg();
	} 
	else if (item instanceof TapestryLocateFailure) {
	    TapestryLocateFailure msg = (TapestryLocateFailure) item;
	    if( lastGuid == null || !msg.obj_guid.equals( lastGuid ) ) {
		if( DEBUG ) {
		    Debug.printtagln( TAG, 
				      "Got incorrect or late failure; " + 
				      "ignoring " + msg );
		}
		return;
	    }
	    boolean timeout = handle_failure( msg );
	    // go on to looking for the next object
	    if( objfind > 0 ) {
		objfind--;
	    }
	    // if this was not a timeout, wait, otherwise we've already waited
	    if( !timeout ) {
		classifier.dispatch_later (new AlarmEvent (), 10000);
	    } else {
		send_new_msg();
	    }
	}
	else {
	    Carp.die (TAG + ": got unknown event of type " + 
		    item.getClass ().getName () + ": " + item + ".");
	}
    }

    public void handleEvents(QueueElementIF items[]) 
    throws EventHandlerException {
	for(int i=0; i<items.length; i++)
	    handleEvent(items[i]);
    }

    public void destroy() throws Exception {

	// print out all the object guids that you published
	// publish all your guids.
	for( int i = objstart; i < objstart+objeach; i++ ) {

	    SecureHash objguid = new SHA1Hash( "object" + i );
	    System.out.println( "DESTROYED " + objguid );
	    
	}

    }

    protected void init_publish() {

	// publish all your guids.
	for( int i = objstart; i < objstart+objeach; i++ ) {

	    SecureHash objguid = new SHA1Hash( "object" + i );
	    TapestryPublishMsg tpm = new TapestryPublishMsg( objguid, null );
	    if((optimize_on & Router.LOCAL_OPT_PUBLISH_BACKUP) != 0) {
		tpm.setLocalOptPublishBackup();
	    }
	    if((optimize_on & Router.LOCAL_OPT_PUBLISH_ALL_LINKS) != 0) {
		tpm.setLocalOptPublishAllLinks();
	    }
	    if((optimize_on & Router.LOCAL_OPT_PUBLISH_LOCAL_MISROUTE) != 0) {
		tpm.setLocalOptPublishLocalMisroute();
	    }
	    if( DEBUG ) {
		System.out.println( node_id + ": publishing object: " + i 
				    + " guid " + objguid );
	    }
	    published_guids.add( objguid );
	    dispatch( tpm );
	    
	}

    }

    protected void got_resp( Response msg ) {
		
	if (msg != null) {
	    long currtime = Clock.current_date( node_id ).getTime();
	    long ping_time = currtime - msg.time;
	    long locate_time = msg.time - lastTimeSent;
	    double stretch;
	    if( objfind > 0 ) {
		objfind--;
	    }
	    if( ping_time != 0 ) {
		stretch = ((double)locate_time) / ((double)ping_time);
		if( msg.guid.equals( lastGuid ) ) {
		    // we found something, print out the time and the hopcount
		    System.out.println( "FOUND: " + node_id + " found obj " + 
					msg.guid + " on node " + msg.peer + 
					" at time " + currtime + 
					" in time " + locate_time +
					" with hopcount " + msg.hopcount + 
					" with stretch " + stretch + 
					" and ping time " + (ping_time*2) + 
					" index " + lastIndex );
		} else {
		    if( DEBUG ) {
			System.out.println( node_id + ": got duplicate or " + 
					    "unrequested response for " + 
					    msg.guid + ", ignoring." );
		    }
		    return;
		}
	    }
	    // cancel the last timeout
	    classifier.cancel( lastToken );
	}
	
    }

    private void send_new_msg() {

	if( objfind != 0 ) {
	    
	    // find the next one
	    int nextobj;
	    if( stanford_test ) {
		nextobj = objfind-1;
	    } else {
		nextobj = rand.nextInt( objtotal );
	    }
	    SecureHash objguid = new SHA1Hash( "object" + nextobj );

	    if( DEBUG ) {
		System.out.println( node_id + ": looking for obj " +
				    objguid );
	    }	    

	    lastTimeSent = Clock.current_date( node_id ).getTime();
	    lastGuid = objguid;
	    lastIndex = nextobj;
	    LocateMsg lm = new LocateMsg( objguid, node_id );
	    if((optimize_on & Router.LOCAL_OPT_PUBLISH_LOCAL_MISROUTE) != 0) {
		lm.setLocalOptLocalMisroute();
	    }
	    dispatch( lm );
	    // if it takes more than a minute, timeout
	    lastToken = 
		classifier.dispatch_later( new TapestryLocateFailure(self_guid,
								     self_guid,
								     objguid,
								     node_id,
								     0 ),
					   10000 );
	}

    }

    private boolean handle_failure( TapestryLocateFailure msg ) {
	boolean timeout = false;
	if( msg.root.equals(self_guid) && msg.id.equals(self_guid) ) {
	    timeout = true;
	}
	if( msg.obj_guid.equals( lastGuid ) ) {
	    long currtime = Clock.current_date( node_id ).getTime();
	    // we found something, print out the time and the hopcount
	    System.out.println( "FAILED: " + node_id + " failed to find obj " 
				+ msg.obj_guid + " on node " + msg.peer + 
				" rootguid " + msg.root + " at time " + 
				currtime + " object index " +
				lastIndex + " timeout " + 
				timeout );
	} else {
	    if( DEBUG ) {
		System.out.println( node_id + ": got duplicate or " + 
				    "unrequested failure for " + 
				    msg.obj_guid + ", ignoring." );
	    }
	}
	// cancel the last timeout
	classifier.cancel( lastToken );
	return timeout;
    }

    /**
     * An internal method used for dispatching events
     */
    private void dispatch( QueueElementIF event ) {

	if( DEBUG ) {
	    System.out.println( node_id + ": Dispatching " + event );
	}
                                                
      try {
        classifier.dispatch( event );
      } catch ( SinkFullException sfe ) {
        //TODO: Better way to deal with SinkFull
        try {
          Thread.currentThread().sleep( 5000 );
        } catch( InterruptedException ie ) {
          //ignore
        }
      }                                         
      
    }

    public static class Response extends NetworkMessage 
	implements QuickSerializable {

	int hopcount;
	long time;
	SecureHash guid;

	public Response (NodeId peer, int hopcount, long time, 
			 SecureHash guid) {
	    
	    super (peer, false /* outbound */);
	    this.hopcount = hopcount;
	    this.time = time;
	    this.guid = guid;
	}

	public Response (InputBuffer buffer) throws QSException {
	    super( buffer );
	    hopcount = buffer.nextInt();
	    time = buffer.nextLong();
	    guid = (SecureHash) buffer.nextObject();
	}

	public void serialize (OutputBuffer buffer) {
	    super.serialize( buffer );
	    buffer.add( hopcount );
	    buffer.add( time );
	    buffer.add( guid );
	}

	public Object clone () throws CloneNotSupportedException {
	    Response rt = (Response) super.clone();
	    rt.hopcount  = hopcount;
	    rt.time = time;
	    rt.guid = guid;
	    return rt;
	}
	
	public String toString () {
	    return "(Response hopcount=" + hopcount + ", time=" + time + 
		", guid=" + guid + "), (" + super.toString () + ")";
	}
    }

    public static class LocateMsg extends TapestryLocateMsg {

	NodeId peer;

	public LocateMsg(SecureHash id, NodeId peer ) {
	    super(id, null, null, null);
	    this.peer = peer;
	    this.id = id;
	}
	
	public LocateMsg (InputBuffer buffer) throws QSException {
	    //super( buffer );
	    peer = (NodeId) buffer.nextObject();
	    id = (SecureHash) buffer.nextObject();
	}

	public void serialize (OutputBuffer buffer) {
	    //super.serialize( buffer );
	    buffer.add( peer );
	    buffer.add( id );
	}

	public Object clone () throws CloneNotSupportedException {
	    LocateMsg rt = (LocateMsg) super.clone();
	    rt.peer = peer;
	    rt.id = id;
	    return rt;
	}
       
	public String toString () {
	    return "(LocateMsg, peer=" + peer + 
		"), (" + super.toString () + ")";
	}

    }

}
