/**
 * PatchworkRevCountMsg.java
 *
 * @author Jeremy Stribling
 * @version    $Id: PatchworkRevCountMsg.java,v 1.2 2002/09/20 07:33:20 hweather Exp $
 */

package ostore.tapestry.patchwork;
import ostore.network.NetworkMessage;
import ostore.util.ByteUtils;
import ostore.util.NodeId;
import ostore.util.InputBuffer;
import ostore.util.OutputBuffer;
import ostore.util.QuickSerializable;
import ostore.util.QSException;

/**
 * This is mostly for testing, to inform other nodes of how many
 * nodes are pointing at me.
 *
 * @author Jeremy Stribling
 * @version    $Id: PatchworkRevCountMsg.java,v 1.2 2002/09/20 07:33:20 hweather Exp $
 */
public class PatchworkRevCountMsg extends NetworkMessage 
    implements QuickSerializable {

    public int num_ptrs;
    // if level == -1, num_ptrs is for all levels
    public int level;

    public PatchworkRevCountMsg (NodeId peer, int num_ptrs, int level) {

	super (peer, false /* outbound */);

	this.num_ptrs = num_ptrs;
	this.level = level;

    }

    public PatchworkRevCountMsg (NodeId peer, int num_ptrs) {
	this( peer, num_ptrs, -1 );	
    }

    public PatchworkRevCountMsg (InputBuffer buffer) throws QSException {
	super( buffer );
	num_ptrs = buffer.nextInt();
	level = buffer.nextInt();
    }

    public void serialize (OutputBuffer buffer) {
	super.serialize(buffer);
	buffer.add( num_ptrs );
	buffer.add( level );
    }

    public Object clone () throws CloneNotSupportedException {
	PatchworkRevCountMsg newone = (PatchworkRevCountMsg) super.clone ();
	newone.num_ptrs = num_ptrs;
	newone.level = level;
	return newone;
    }

    public String toString () {
	return "(PatchworkRevCountMsg, num: " + 
		num_ptrs + ", level: " + level + " "
		+ super.toString () + ")";
    }
}

