/**
 * RepairMonitorBuilder.java
 *
 * Copyright (c) 2001 Regents of the University of California.
 * All rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *  3. Neither the name of the University nor the names of its contributors
 *     may be used to endorse or promote products derived from this software
 *     without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */

package ostore.tapestry.patchwork;

import soss.core.BuilderIF;
import ostore.util.Carp;
import seda.sandStorm.main.SandstormConfig;
import seda.sandStorm.api.StageNameAlreadyBoundException;
import java.io.IOException;
import java.util.StringTokenizer;

/**
 * A configuration builder for RepairMonitors
 * Modelled after <code>ostore.bloom.DualBuilder</code>.
 *
 * @author	Jeremy Stribling
 * @version	$Id: RepairMonitorBuilder.java,v 1.5 2002/12/20 23:54:24 strib Exp $
 */
public class RepairMonitorBuilder implements BuilderIF
{
  ////////////////////       CONSTANTS	////////////////////
  ////////////////////    STATIC VARIABLES 	////////////////////
  ////////////////////       VARIABLES	////////////////////
  ////////////////////  STATIC CONSTRUCTORS	////////////////////
  ////////////////////      CONSTRUCTORS	////////////////////

  String fed_node;

  /**
   * Set global parameters. */
  public RepairMonitorBuilder(String[] args)
  {
    if (args.length != 1) 
      Carp.die ("usage: RepairMonitorBuilder <fed_node>");
	
    fed_node = args[0];
  }

  ////////////////////     STATIC METHODS 	////////////////////
  //////////////////// 	METHODS		////////////////////

  /**
   * Create a configuration object for a single virtual machine. */
  public SandstormConfig build_node(String[] args)
      throws StageNameAlreadyBoundException, IOException
  {
      if(args.length != 36)
    {
      String str = new String();
      for(int i = 0; i < args.length; i++)
	  str += args[i] + " ";
      Carp.die ("usage: RepairMonitorBuilder.build_node <node_id>" +
		" <num_failures> <random_seed>" +
		" <fail_and_replace?> <fail_immed?> <allow_parallel>" +
		" <use_old_guid> <verify_tapestry>" +
		" <cdf_class> <life_cdf_class> <gateway> <time_to_run>" +
		" <msgtowatch1_msgtowatch2_etc>" +
		" <repairTapestry?> <basePeriod>" + 
		" <op> <factor> <patchworkTapestry?> <respond_period>" +
		" <loss_period> <loss_op> <loss_factor> <latency_period>" + 
		" <latency_op> <latency_factor> <bandwidth_period>" +
		" <bandwidth_op> <bandwidth_factor> <recalc_period>" +
		" <second_chances_thresh> " +
		" <second_chances_resp> <timeout_packets> <repair_timeout>" +
		" <cache_size> <cache_dir> <publish>");
    }

    String node_id = args[0];
    String num_failures = args[1];
    String random_seed = args[2];
    String fail_and_replace = args[3];
    String fail_immed = args[4];
    String allow_parallel = args[5];
    String use_old_guid = args[6];
    String verify_tapestry = args[7];
    String cdf_class = args[8];
    String life_cdf_class = args[9];
    String gateway = args[10];
    String time_to_run = args[11];
    String msgs_args = args[12];

    boolean repairTapestry = (new Boolean(args[13])).booleanValue();
    String basePeriod = args[14];
    String op = args[15];
    String factor = args[16];
    
    boolean patchworkTapestry = (new Boolean(args[17])).booleanValue();
    String respond_period = args[18];
    String loss_period = args[19];
    String loss_op = args[20];
    String loss_factor = args[21];
    String latency_period = args[22];
    String latency_op = args[23];
    String latency_factor = args[24];
    String bandwidth_period = args[25];
    String bandwidth_op = args[26];
    String bandwidth_factor = args[27];
    String recalc_period = args[28];
    String second_chances_thresh = args[29];
    String second_chances_resp = args[30];
    String timeout_packets = args[31];
    String repair_timeout = args[32];
    
    String cacheSize = args[33];
    String cacheDir = args[34];
    String publish = args[35];

    StringTokenizer msgsArgsST = new StringTokenizer( msgs_args, "_" );
    
    SandstormConfig result = new SandstormConfig();
    
    // Router
    {
      String[] init_args = {
	  "node_id=" + node_id,
	  "dynamic_route=static",
	  "fake_keys=true"		
        };
      result.addStage("Router", 
		      "ostore.tapestry.impl.Router",
		      init_args);
    }

    // StaticTClient
    {
      String[] init_args = {
	  "node_id=" + node_id,
	  "dynamic_route=static",
	  "federation=" + fed_node
        };
      result.addStage("StaticTClient", 
		      "ostore.tapestry.impl.StaticTClient",
		      init_args);
    }

    // DynamicTClient
    {
	String[] init_args = {
	    "node_id=" + node_id,
	    "dynamic_route=static",
	    "gateway=" + gateway
	};
	result.addStage("DynamicTClient", 
			"ostore.tapestry.impl.DynamicTClient",
			init_args);
    }

    if( repairTapestry ) {
	// RepairStage
	{
	    String[] init_args = {
		"node_id=" + node_id,
		"DataDrivenServerHbBasePeriod=" + basePeriod,
		"DataDrivenServerHbOp=" + op,
		"DataDrivenServerHbFactor=" + factor
	    };
	    result.addStage("RepairStage",
			    "ostore.archive.repair.RepairStage",
			    init_args);
	}
	
	// Cache
	{
	    String[] init_args = {
		"node_id=" + node_id,
		"CacheSize=" + cacheSize,
		"cachedir=" + cacheDir,
		"Publish=" + publish,
		"one_cache=true"
	    };
	    result.addStage("MemoryCache",
			    "ostore.cache.MemoryCache",
			    init_args);
	}
	
    }
    
    if( patchworkTapestry ) {
	// Patchwork
	{
	    String[] init_args = {
		"node_id=" + node_id,
		"loss_period=" + loss_period,
		"loss_op=" + loss_op,
		"loss_factor=" + loss_factor,
		"latency_period=" + latency_period,
		"latency_op=" + latency_op,
		"latency_factor=" + latency_factor,
		"bandwidth_period=" + bandwidth_period,
		"bandwidth_op=" + bandwidth_op,
		"bandwidth_factor=" + bandwidth_factor,
		"recalc_period=" + recalc_period,
		"respond_period=" + respond_period,
		"second_chances_thresh=" + second_chances_thresh,
		"second_chances_resp=" + second_chances_resp,
		"timeout_packets=" + timeout_packets,
		"optimize_on=true",
		"accept_faults=true"
	    };
	    result.addStage("PatchworkStage", 
			    "ostore.tapestry.patchwork.PatchworkStage",
			    init_args);
	}
    }
    
    // PatchworkSimulation 
    {
	
	String[] init_args = {
	    "node_id=" + node_id,
	    "deletable=false"
	};
	result.addStage("PatchworkSimulationStage", 
			"ostore.tapestry.patchwork.PatchworkSimulationStage",
			init_args);
    }
    
    // RepairMonitor
    {
	
	String messages = " ";
	while( msgsArgsST.hasMoreTokens() ) {
	    messages += msgsArgsST.nextToken() + " ";
	}
	
	String[] init_args = {
	    "node_id=" + node_id,
	    "num_failures=" + num_failures,
	    "random_seed=" + random_seed,
	    "fail_and_replace=" + fail_and_replace,
	    "fail_immed=" + fail_immed,
	    "allow_parallel=" + allow_parallel,
	    "use_old_guid=" + use_old_guid,
	    "verify_tapestry=" + verify_tapestry,
	    "cdf_class=" + cdf_class,
	    "life_cdf_class=" + life_cdf_class,
	    "time_to_run=" + time_to_run,
	    "messages=" + messages,
	    "repair_timeout=" + repair_timeout
	    
	};
	result.addStage("RepairMonitorStage", 
			"ostore.tapestry.patchwork.RepairMonitorStage",
			init_args);
    }
    
    return result;
  }  
  ////////////////////     INNER CLASSES	////////////////////
}

