/*
 *  Copyright (c) 2001 Regents of the University of California.  All
 *  rights reserved.
 *
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above
 *     copyright notice, this list of conditions and the following
 *     disclaimer in the documentation and/or other materials provided
 *     with the distribution.
 *  3. Neither the name of the University nor the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 *  IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 *  FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 *  REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 *  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 *  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 *  OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 *  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ostore.util;

import java.util.*;

/**
 * Map where (key =&gt; Set&lt;values&gt;)
 */
public class MultiMap
{
    private Map map;
    private Class setType;

    public MultiMap()
    {
	map = new HashMap();
	setType = HashSet.class;
    }

    public MultiMap(int initialCapacity)
    {
	map = new HashMap(initialCapacity);
	setType = HashSet.class;
    }

    public MultiMap(int initialCapacity, Class setType)
    {
	map = new HashMap(initialCapacity);
	this.setType = setType;
    }

    public void put(Object key, Object value)
    {
	Set s = (Set)map.remove(key);
	if (s == null) {
	    try {
		s = (Set)setType.newInstance();
	    } catch (Exception ex) {
		ex.printStackTrace();
		Carp.die(ex.toString());
	    }
	}

	s.add(value);
	map.put(key, s);
    }

    public Set get(Object key)
    {
	return (Set)map.get(key);
    }

    public boolean containsKey(Object key)
    {
	return map.containsKey(key);
    }

    public Set remove(Object key)
    {
	return (Set)map.remove(key);
    }
    
    public Set keySet()
    {
	return map.keySet();
    }

    /**
     * Collection of the inividual values.
     */
    public Collection values()
    {
	Set result = null;
	try {
	    result = (Set)setType.newInstance();
	} catch (Exception ex) {
	    ex.printStackTrace();
	    Carp.die(ex.toString());
	}

	for (Iterator i=map.values().iterator(); i.hasNext(); ) {
	    Set s = (Set)i.next();
	    for (Iterator j=s.iterator(); j.hasNext(); ) {
		result.add(j.next());
	    }
	}

	return result;
    }
    
    /**
     * Number of values in map.
     */
    public int size()
    {
	return values().size();
    }

    /**
     * Number of keys in map.
     */
    public int numKeys()
    {
	return map.size();
    }
}
