#!/usr/bin/perl -w
#
# $Id: diff_app.pl,v 1.1 2003/01/16 23:04:45 geels Exp $

use strict;

my $first_file;
my @first_lines;
my @first_hosts;
my %first_times;	# host x host -> RTT map

my $second_file;
my @second_lines;
my @second_hosts;
my %second_times;	# host x host -> RTT map

####################
# Slurp in the two files

($first_file, $second_file) = @ARGV;
print "Comparing $first_file to $second_file:\n";

open( FIRST, "<$first_file" ) or die "Cannot open $first_file\n";
@first_lines = <FIRST>;
close( FIRST ) or die "Cannot close $first_file\n";
splice @first_lines, 0, 2;
#print "Number lines in $first_file: $#first_lines\n";

open( SECOND, "<$second_file" ) or die "Cannot open $second_file\n";
@second_lines = <SECOND>;
close( SECOND ) or die "Cannot close $second_file\n";
splice @second_lines, 0, 2;
#print "Number lines in $second_file: $#second_lines\n";

my $line = shift( @first_lines );
my $host;
my $other;
@first_hosts = split( /\s+/, $line );
#print "Number hosts in $first_file: " . scalar(@first_hosts) . "\n";
for( my $i=0; $i<@first_hosts; $i++ ) {
    $line = $first_lines[$i];
    my @times = split( /\s+/, $line );
#    print "Number times in line #$i: " . scalar(@times) . "\n";
#    print "First few: \'$times[0]\', \'$times[1]\'\n"; 
    $host = $first_hosts[$i];
    for( my $j=0; $j<@first_hosts; $j++ ) {
	$other = $first_hosts[$j];
	$times[$j] =~ m%.*/(.*)/.*%;
	$first_times{$host}{$other} = $1;
    }
}

$line = shift( @second_lines );
@second_hosts = split( /\s+/, $line );
#print "Number hosts in $second_file: " . scalar(@second_hosts) . "\n";
for( my $i=0; $i<@second_hosts; $i++ ) {
    $line = $second_lines[$i];
    my @times = split( /\s+/, $line );
#    print "Number times in line #$i: " . scalar(@times) . "\n";
#    print "Second few: \'$times[0]\', \'$times[1]\'\n"; 
    $host = $second_hosts[$i];
    for( my $j=0; $j<@second_hosts; $j++ ) {
	$other = $second_hosts[$j];
	$times[$j] =~ m%.*/(.*)/.*%;
	$second_times{$host}{$other} = $1;
    }
}

####################
# Now print gnuplot data files with the diff

my $datafile = "diff_app.data";
open( DATA, ">$datafile" ) or die "Cannot open $datafile\n";

my $means_file = "diff_app.means";
open( MEANS, ">$means_file" ) or die "Cannot open $means_file\n";

my $new_file = "diff_app.new";
open( NEW, ">$new_file" ) or die "Cannot open $new_file\n";
print "Printing $datafile, $means_file, $new_file\n";

my @good_hosts;
foreach $host (sort keys %first_times) {
    if( !defined $second_times{$host} ) {
	print "Host $host not defined in second file.  Skipping.\n";
	next;
    }
    push @good_hosts, $host;
    my %subhash = %{$first_times{$host}};
    my @diffs;
    my @times;
    foreach $other (sort keys %subhash) {
	next if( !defined $second_times{$other} );	
#	print "$host to $other = $first_times{$host}{$other}\n";
	my $diff = $second_times{$host}{$other} - $first_times{$host}{$other};
	print DATA "$diff ";
	push @diffs, $diff;
	if( abs( $diff ) > 25 ) {
	    print "$host to $other \tchanged by $diff \t(" .
		$first_times{$host}{$other} . " to ".
		$second_times{$host}{$other} . ")\n";
	}
	push @times, $second_times{$host}{$other};
    }
    print DATA "\n";
    my $sum = 0;
    foreach( @diffs ) { $sum += $_ };
    my $mean = $sum/@diffs;
    $sum = 0;    
    foreach( @diffs ) { $sum += ($_-$mean)*($_-$mean) };
    my $stddev = sqrt( $sum / $#diffs );
    if( abs($mean) > 5 ) {
	print "$host mean: $mean stddev: $stddev\n";
    }
    print MEANS "$mean $stddev\n";

    $sum = 0;
    foreach( @times ) { $sum += $_ };
    $mean = $sum/@times;
    $sum = 0;    
    foreach( @times ) { $sum += ($_-$mean)*($_-$mean) };
    $stddev = sqrt( $sum / $#times );
    print NEW "$mean $stddev\n";

}
foreach $host (sort keys %second_times) {
    if( !defined $first_times{$host} ) {
	print "Host $host not defined in first file.  Skipping.\n";
    }
}
close( DATA ) or die "Cannot close $datafile\n";
close( MEANS ) or die "Cannot close $means_file\n";
close( NEW ) or die "Cannot close $means_file\n";

##########
# 3-d diff matrix.
my $gnuplot_file = "diff_app.gnuplot";
my $data_eps = "diff_app.eps";
open( GNUPLOT, ">$gnuplot_file" ) or die "Cannot open $gnuplot_file\n";
print GNUPLOT <<HERE;
set terminal postscript eps "Times-Roman" 26
set output "$data_eps"
splot "$datafile" matrix with impulses
HERE
close( GNUPLOT ) or die "Cannot close $gnuplot_file\n";

system( "gnuplot", $gnuplot_file );
system( "gv $data_eps &" );

##########
# Diffs for each host
my $xmax = @good_hosts+1;
my $data_eps2 = "diff_app2.eps";
open( GNUPLOT, ">$gnuplot_file" ) or die "Cannot open $gnuplot_file\n";
for( my $i=0; $i<@good_hosts; $i++ ) {
    $host = $good_hosts[$i];
    print GNUPLOT "set label \"$host\" at graph " . ((0.5+$i)/($xmax+1)) .
	",-0.2 rotate font \"Times-Roman,10\"\n";
}
print GNUPLOT <<HERE;
set terminal postscript eps "Times-Roman" 26
set output "$data_eps2"
set noxtics
set lmargin 4
set bmargin 3
set xrange [-1:$xmax]
plot "$means_file" using column(-1):(\$1):(\$2) notitle with errorbars, "$means_file" using column(-1):(\$1) notitle with lines, 0 notitle
HERE
close( GNUPLOT ) or die "Cannot close $gnuplot_file\n";

system( "gnuplot", $gnuplot_file );
system( "gv -landscape -scale 2 $data_eps2 &" );

##########
# Diffs for each host
my $data_eps3 = "diff_app3.eps";
open( GNUPLOT, ">$gnuplot_file" ) or die "Cannot open $gnuplot_file\n";
for( my $i=0; $i<@good_hosts; $i++ ) {
    $host = $good_hosts[$i];
    print GNUPLOT "set label \"$host\" at graph " . ((0.5+$i)/($xmax+1)) .
	",-0.2 rotate font \"Times-Roman,10\"\n";
}
print GNUPLOT <<HERE;
set terminal postscript eps "Times-Roman" 26
set output "$data_eps3"
set noxtics
set lmargin 4
set bmargin 3
set xrange [-1:$xmax]
plot "$new_file" using column(-1):(\$1):(\$2) notitle with errorbars, "$new_file" using column(-1):(\$1) notitle with lines
HERE
close( GNUPLOT ) or die "Cannot close $gnuplot_file\n";

system( "gnuplot", $gnuplot_file );
system( "gv -landscape -scale 2 $data_eps3" );

unlink $datafile, $means_file, $new_file, $gnuplot_file, $data_eps, $data_eps2, $data_eps3;
