/**
 * AnalyzerMenu.java
 *
 * Copyright (c) 2001 Regents of the University of California.  All
 * rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided.
 *    with the distribution.
 * 3. Neither the name of the University nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package Sonar.PacketAnalyzers;

import Sonar.Dialog.*;
import Sonar.event.MainMenuEvent;
import Sonar.event.OptionsMenuEvent;
import Sonar.event.AnalyzerMenuEvent;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.io.Serializable;

//SANDSTORM
import ostore.util.Carp;
import ostore.util.Debug;
import ostore.util.NodeId;
import ostore.dispatch.Classifier;
import ostore.dispatch.Filter;
import ostore.dispatch.Signal;

import seda.sandStorm.api.EventHandlerIF;
import seda.sandStorm.api.EventHandlerException;
import seda.sandStorm.api.ConfigDataIF;
import seda.sandStorm.api.ProfilableIF;
import seda.sandStorm.api.QueueElementIF;
import seda.sandStorm.api.SinkFullException;
import seda.sandStorm.api.SinkIF;
import seda.sandStorm.internal.SinkProxy;
import seda.sandStorm.api.StagesInitializedSignal;

/**
 * Determines whether to paint an edge for a specific message. This is an   
 * auxiliary class to be used with the DisplayManager, which creates an options
 * menu.
 * 
 * @author	Jason Lee <jlee81@cs.berkeley.edu>
 */

public class AnalyzerMenu extends PacketAnalyzer implements ProfilableIF, EventHandlerIF {
    //dissemination
    public boolean show_create_obj;
    public boolean show_disseminate_frag;
    public boolean show_ir_create; //InnerRing Create
    public boolean show_update;
    public boolean show_req_frag;
    public boolean show_heartbeat;

    public boolean show_ir_traffic;
    public boolean show_imap;
    public boolean show_datadrivenhb;
    public boolean show_patchwork;

    //ICMP
    public boolean show_ICMP; //Traceroutes

    public boolean show_traffic; 
    public boolean show_along_tapestry;

    protected MenuManager menu_manager;

    protected AnalyzerMenu self_pointer = null;

    public void inst() {
    	show_create_obj = true;
	show_disseminate_frag = true;
	show_ir_create = true;
	show_update = true;
	show_req_frag = true;
	show_ir_traffic = true;
	show_imap = true;

	show_ICMP = false;
	show_heartbeat = false;
	show_datadrivenhb = false;
	show_patchwork = false;

	show_traffic = false; //all msgs versus selected ones 
	show_along_tapestry = false;

	menu_manager = new MenuManager();
	self_pointer = this;
    }

    public void showOptionsDialog() {
        StandardDialog newDialog = new StandardDialog(new OptionsPanel());
	newDialog.show();
    }

    public ActivePanel getOptionsPanel() { 
        return new OptionsPanel();
    }

    public class OptionsPanel extends Sonar.Dialog.ActivePanel {
        public OptionsPanel() {
	    tabTitle = "Analyzers";
	    setLayout(null);
	    setSize(377, 268);

	    j_combobox1.addItem(new String("Listened Types, End-to-End"));
	    j_combobox1.addItem(new String("Listened Types, Hop-to-Hop"));
	    //j_combobox1.addItem(new String("All, End-to-End"));
	    //j_combobox1.addItem(new String("All, Hop-to-Hop"));
	    add(j_combobox1);
	    j_combobox1.setBounds(20, 34, 210, 24);

	    j_label1.setText("Edges to be Displayed:");
	    j_label1.setBounds(20, 10, 150, 24);
	    add(j_label1);

	    j_checkbox1.setToolTipText("pink edges");
	    j_checkbox1.setText("create object");
	    add(j_checkbox1);
	    j_checkbox1.setBounds(20, 68, 164, 24);

	    j_checkbox2.setToolTipText("light gray edges");
	    j_checkbox2.setText("inner ring");
	    add(j_checkbox2);
	    j_checkbox2.setBounds(20, 92, 164, 24);
	    
	    j_checkbox3.setToolTipText("red edges");
	    j_checkbox3.setText("disseminate frag");
	    add(j_checkbox3);
	    j_checkbox3.setBounds(20,116, 164, 24);

	    j_checkbox4.setToolTipText("magenta edges");
	    j_checkbox4.setText("update request");
	    add(j_checkbox4);
	    j_checkbox4.setBounds(20, 140, 164, 24);

	    j_checkbox5.setToolTipText("cyan edges");
	    j_checkbox5.setText("request fragment");
	    add(j_checkbox5);
	    j_checkbox5.setBounds(20, 164, 164, 24);
	    
	    j_checkbox6.setToolTipText("(red/blue) msgs");
	    j_checkbox6.setText("trace routes");
	    add(j_checkbox6);
	    j_checkbox6.setBounds(20, 188, 164, 24);

	    j_checkbox7.setToolTipText("yellow edges - hop to hop only");
	    j_checkbox7.setText("publish msgs");
	    add(j_checkbox7);
	    j_checkbox7.setBounds(20,212, 164, 24);

	    j_checkbox8.setToolTipText("gray edges");
	    j_checkbox8.setText("IR traffic");
	    add(j_checkbox8);
	    j_checkbox8.setBounds(184, 92, 100, 24);

	    j_checkbox9.setToolTipText("orange edges");
	    j_checkbox9.setText("imap");
	    add(j_checkbox9);
	    j_checkbox9.setBounds(184, 116, 140, 24);

	    j_checkbox10.setToolTipText("yellow edges");
	    j_checkbox10.setText("heartbeat");
	    add(j_checkbox10);
	    j_checkbox10.setBounds(184, 140, 140, 24);

	    j_checkbox11.setToolTipText("yellow edges");
	    j_checkbox11.setText("patchwork");
	    add(j_checkbox11);
	    j_checkbox11.setBounds(184,164, 140, 24);

	}
	javax.swing.JCheckBox j_checkbox1 = new javax.swing.JCheckBox();
	javax.swing.JCheckBox j_checkbox2 = new javax.swing.JCheckBox();
	javax.swing.JCheckBox j_checkbox3 = new javax.swing.JCheckBox();
	javax.swing.JCheckBox j_checkbox4 = new javax.swing.JCheckBox();
	javax.swing.JCheckBox j_checkbox5 = new javax.swing.JCheckBox();
	javax.swing.JCheckBox j_checkbox6 = new javax.swing.JCheckBox();
	javax.swing.JCheckBox j_checkbox7 = new javax.swing.JCheckBox();
	javax.swing.JCheckBox j_checkbox8 = new javax.swing.JCheckBox();
	
	javax.swing.JCheckBox j_checkbox9 = new javax.swing.JCheckBox();
	javax.swing.JCheckBox j_checkbox10 = new javax.swing.JCheckBox();
	javax.swing.JCheckBox j_checkbox11 = new javax.swing.JCheckBox();
	
	javax.swing.JLabel j_label1 = new javax.swing.JLabel();
	javax.swing.JComboBox j_combobox1 = new javax.swing.JComboBox();

	public void applyChanges() {
	    show_create_obj = j_checkbox1.isSelected();
	    show_ir_create = j_checkbox2.isSelected(); 
	    show_disseminate_frag = j_checkbox3.isSelected();
	    show_update = j_checkbox4.isSelected();
	    show_req_frag = j_checkbox5.isSelected();
	    show_ICMP = j_checkbox6.isSelected();
	    show_heartbeat = j_checkbox7.isSelected();
	    show_ir_traffic= j_checkbox8.isSelected();

	    show_imap = j_checkbox9.isSelected();
	    show_datadrivenhb = j_checkbox10.isSelected();
	    show_patchwork = j_checkbox11.isSelected();

	    if(j_combobox1.getSelectedItem().equals("Listened Types, End-to-End")) {
	        show_traffic = false;
		show_along_tapestry = false;
	    } else if(j_combobox1.getSelectedItem().equals("Listened Types, Hop-to-Hop")) {
	        show_traffic = false;
		show_along_tapestry = true;
	    } else if(j_combobox1.getSelectedItem().equals("All, End-to-End")) {
		show_traffic = true;
		show_along_tapestry = false;
	    } else if(j_combobox1.getSelectedItem().equals("All, Hop-to-Hop")) {
	        show_traffic = true;
		show_along_tapestry = true;
 	    }

	    
	    try {
		_classifier.dispatch(new AnalyzerMenuEvent(self_pointer));
	    } catch (SinkFullException e) {
		e.printStackTrace();
	    }
	}

	public void initializeDisplayValues() {
	    j_checkbox1.setSelected(show_create_obj);
	    j_checkbox2.setSelected(show_ir_create);
	    j_checkbox3.setSelected(show_disseminate_frag);
	    j_checkbox4.setSelected(show_req_frag);
	    j_checkbox5.setSelected(show_req_frag);
	    j_checkbox6.setSelected(show_ICMP);
	    j_checkbox7.setSelected(show_heartbeat);
	    j_checkbox8.setSelected(show_ir_traffic);
	    j_checkbox9.setSelected(show_imap);
	    j_checkbox10.setSelected(show_datadrivenhb);
	    j_checkbox11.setSelected(show_patchwork);

	    if(!show_traffic && !show_along_tapestry) {
	        j_combobox1.setSelectedItem(new String("Listened Types, End-to-End"));
	    } else if(!show_traffic && show_along_tapestry) {
	        j_combobox1.setSelectedItem(new String("Listened Types, Hop-to-Hop"));
	    } else if(show_traffic && !show_along_tapestry) {
		j_combobox1.setSelectedItem(new String("All, End-to-End"));
	    } else {
	        j_combobox1.setSelectedItem(new String("All, Hop-to-Hop"));
	    }
	}
    }

    protected class MenuManager implements Serializable, ActionListener {
        JMenu main_menu = new JMenu();
	JMenuItem properties_item = new JMenuItem();
	JMenuItem legend = new JMenuItem();


	public MenuManager() {
	    main_menu.setText("Analyzer Menu");
	    main_menu.setActionCommand("Analyzer Menu");

	    properties_item.setText("Options");
	    properties_item.setActionCommand("Options");
	    main_menu.add(properties_item);

	    properties_item.addActionListener(this);
	}

	public void actionPerformed(ActionEvent e) {
	    Object object = e.getSource();
	    if(object == properties_item)
	        showOptionsDialog();
	}
    }
    /**
     * SANDSTORM 
     */
    private static final String class_tag = "AnalyzerMenu";
    private static final boolean DEBUG = false;

    public void handleEvent (QueueElementIF item) throws EventHandlerException {
	final String tag = class_tag + ".handleEvent";
	if (item instanceof StagesInitializedSignal) {
	    try {
		if(DEBUG) Debug.printtagln(tag, "registering analyzer menu as a panel contributor");
		_classifier.dispatch(new MainMenuEvent(menu_manager.main_menu));
		_classifier.dispatch(new OptionsMenuEvent(this));

	    } catch (SinkFullException e) {
		Debug.printtagln(tag, "failed to register AM as an options menu contributor.");
	    }
	}
    }

    public void handleEvents (QueueElementIF items[]) throws EventHandlerException {
	final String tag = class_tag + ".handleEvents";
	for(int i=0; i<items.length; i++ ) 
	    handleEvent(items[i]);
    }

    public void init(ConfigDataIF config) throws Exception {
	final String tag = class_tag + ".init";

	Filter filter = null;
	String messageTypes[] = {"seda.sandStorm.api.StagesInitializedSignal"};

	if (DEBUG) Debug.turnOn(class_tag);
	if (DEBUG) Debug.printtagln (tag, "Debug turned on ");

	_self_node_id = new NodeId(config.getString("node_id"));
	_classifier = Classifier.getClassifier ( _self_node_id);

	stageName = config.getStage().getName();
	if(config.getBoolean("Profile")) {
	    config.getManager().getProfiler().add("Sonar.PacketAnalyzers.AnalyzerMenu", this);
	    SinkIF sink = config.getStage().getSink();
	    _this_sink = new SinkProxy(sink, config.getManager(), 
		    config.getStage().getWrapper());
	} else 
	    _this_sink = config.getStage().getSink();

	for(int i = 0; i < messageTypes.length; i++) {
	    filter = new Filter();
	    if(!filter.requireType(Class.forName (messageTypes[i])))
		BUG ( tag + ": could not require type==" +
			messageTypes[i] + ".");
	    _classifier.subscribe (filter, _this_sink);
	}

	inst();
	//Retrieve all booleans, settings from the sonar.cfg
	
    }

    // methods for ProfilableIF
    public int profileSize()
    {
	return _this_sink.size();
    }
    // end methods for ProfilableIF
    
    // methods for EventHandlerIF
    public void destroy () throws Exception { }
    // end methods for EventHandlerIF
    
    private static final void BUG (String msg) {
	Carp.die (msg + " This is a bug in " + class_tag + ". " +
		"Please let Jason know about it ASAP." );
    }

    private String stageName;
    private Classifier _classifier;
    private SinkIF _this_sink;
    private NodeId _self_node_id;

}
