/**
 * EdgeInfo.java
 *
 * Copyright (c) 2002 Regents of the University of California.  All
 * rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the University nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
package Sonar.util; 
import ostore.util.NodeId;

/**
 * @author	Jason Lee <jlee81@cs.berkeley.edu>
 * @author	Kamin Whitehouse <kamin@cs.berkeley.edu>
 */

public class EdgeInfo implements java.io.Serializable {
    protected NodeId sourceNodeId; 
    protected NodeId destinationNodeId;
    protected boolean bidirectional = false;
    protected String type;

    protected double distance;
    protected boolean displayLength;

    public int instances = 0;
    protected long lastUpdatedTime;
    protected NodeId origin;

    /**
     * Used by grid location analyzer
     */
    public EdgeInfo(NodeId source, NodeId dest) {
	sourceNodeId = source;
	destinationNodeId = dest;
	instances = 1;
	type = "no type";

	distance = 0.5;
	displayLength = false;
    }

    public EdgeInfo(NodeId source, NodeId dest, String t) {
	sourceNodeId = source; 
	destinationNodeId = dest; 
	instances = 1;
	type = t;

	distance = 0.5;
	displayLength = false;
    }
    
    public EdgeInfo(NodeId source, NodeId dest, String t, long time) {
	sourceNodeId = source;
	destinationNodeId = dest;
	instances = 1;
	lastUpdatedTime = time;
	type = t;

	distance = 0.5;
	displayLength = false;
    }

    public String getType() { return type; }

    public boolean isBidirectional() { return bidirectional; }
    public void setBidirectional(boolean bothDirections) {
	bidirectional = bothDirections;
    }

    public NodeId getSource(){return sourceNodeId;}		
    public NodeId getDest(){return destinationNodeId;}	
    public void setSource(NodeId src) {
	sourceNodeId = src;
    }
    public void setDest(NodeId dest) {
	destinationNodeId = dest;
    }

    public int getInstances() { return instances; }
    public void setInstances(int num) { instances = num; }

    public void modifyTime(long time) { lastUpdatedTime = time; }
    public long getTime() { return lastUpdatedTime; }

    public double getDistance(){return distance;} 
    public void setDistance(double pDistance){distance = pDistance;} 
    public boolean getDisplayLength(){return displayLength;} 
    public void setDisplayLength(boolean pDisplayLength){displayLength = pDisplayLength;} 

    public void setOrigin(NodeId src) {
	origin = src;
    }
    public NodeId getOrigin() {
	return origin;
    }
}
