/**
 * LocationAnalyzer.java
 *
 * Copyright (c) 2002 Regents of the University of California.  All
 * rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the University nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package Sonar.util;

import java.util.*;
import ostore.util.NodeId;

/**
 * This class serves as an "adapter" (i.e. an interface with methods).
 *
 * @author	Kamin Whitehouse <kamin@cs.berkeley.edu>
 */
public interface LocationAnalyzer
{

  public double getCenterX(); 
  public double getCenterY();
  public void paintGrids(int x, int y, int radius, java.awt.Graphics g);

    public double getX(NodeId nodeId);
    public double getY(NodeId nodeId);
    public void setX(NodeId nodeNumber, double pX);
    public void setY(NodeId nodeNumber, double pY);
    public double getDistance(NodeId sourceNode,
	    NodeId destinationNode);
    public void setDistance(NodeId sourceNodeId,
	    NodeId destinationNodeId, double pDistance);
    public NodeId findNearestNode(double x, double y);
    public Vector findAllNodesWithinRectangle(double x, double y1, double x1, double y2);
    public Vector findNearestEdge(double x, double y);
    public void identifyNode(NodeId node, int type);
}
