/*
 * SimulatorSink.java
 *
 * Copyright (c) 2001 Regents of the University of California.  All
 * rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. Neither the name of the University nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS
 * IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
 * REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package soss.core;
import java.util.HashMap;
import java.util.Map;
import ostore.util.Carp;
import seda.sandStorm.api.SinkIF;
import seda.sandStorm.api.QueueElementIF;
import seda.sandStorm.api.EventHandlerIF;
import seda.sandStorm.api.EnqueuePredicateIF;
import seda.sandStorm.api.SinkException;
import seda.sandStorm.api.SinkFullException;
import seda.sandStorm.api.EventHandlerException;

/**
 * Part of faking out Sandstorm.
 *
 * @author Sean C. Rhea
 * @version $Id: SimulatorSink.java,v 1.5 2003/06/18 21:07:34 srhea Exp $
 */
public class SimulatorSink implements SinkIF {

    private static Integer _next_hash_code_lock = new Integer (42);
    private static int _next_hash_code = 0;
    private int _hashcode;

    public int hashCode () {
	return _hashcode;
    }

    public SimulatorSink (EventHandlerIF handler) {
	_next_key = 0;
	_prepares = new HashMap ();
	_handler = handler;
        if (handler == null) 
            Carp.die ("SimulatorSink: handler must be non-null");
	synchronized (_next_hash_code_lock) {
	    _hashcode = _next_hash_code++;
	}
    }

    public void enqueue (QueueElementIF event) throws SinkException {
	try {
	    _handler.handleEvent (event);
	}
	catch (EventHandlerException e) {
	    throw new SinkFullException ("from EventHandlerException");
	}
    }

    public synchronized boolean enqueue_lossy (QueueElementIF event) {
	try {
	    enqueue (event);
	    return true;
	} catch (SinkException e) {
	    return false;
	}
    }

    public synchronized void enqueue_many (QueueElementIF [] events) 
    throws SinkException {
	for (int i = 0; i < events.length; i++) {
	    enqueue (events [i]);
	}
    }

    public Object enqueue_prepare (QueueElementIF [] events) 
    throws SinkException {
	// TODO: ghetto
	Long key = new Long (_next_key++);
        // System.out.println ("enqueue_prepare(" + key + ")");
	_prepares.put (key, events);
	return key;
    }

    public void enqueue_commit (Object key) {
        // System.out.println ("enqueue_commit(" + key + ")");
	QueueElementIF [] events = (QueueElementIF []) _prepares.remove (key);
        if (events == null)
            throw new IllegalArgumentException (
                    "enqueue_commit called with unknown key: " + key);
	try {
	    enqueue_many (events);
	}
	catch (SinkException e) {
	    System.err.println ("SimulatorSink: Ack!");
	    System.exit (1);
	}
    }

    public void enqueue_abort(Object key) {
        // System.out.println ("enqueue_abort(" + key + ")");
    }

    public void setEnqueuePredicate (EnqueuePredicateIF pred) {
	// TODO
	throw new IllegalArgumentException ("setEnqueuePredicate not " +
		"supported by SimulatorSink.");
    }

    public EnqueuePredicateIF getEnqueuePredicate () {
	// TODO
	return null;
    }

    public int size () {
	// TODO
	return 0;
    }

    public String toString () {
	return "soss.core.SimulatorSink";
    }

    public EventHandlerIF getHandler() {
	return _handler;
    }

    private long _next_key;
    private Map _prepares;
    private EventHandlerIF _handler;
}

